@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -44,20 +44,23 @@ const char kProductNameLocalized[] = "Mozc";
 #endif  // GOOGLE_JAPANESE_INPUT_BUILD
 
 #if defined(OS_WINDOWS)
-const char kCompanyNameInEnglish[] = "Google";
-const char kCharacterPadMailslotPrefix[]
-    = "\\\\.\\mailslot\\googlejapaneseinput.character_pad.";
 #ifdef GOOGLE_JAPANESE_INPUT_BUILD
+// Most of constants for Windows have not been branding-aware yet.
+
+// Safe length of IME name in terms of IME_ESC_IME_NAME request.
+// See http://msdn.microsoft.com/en-us/library/dd318166.aspx for details.
+const int kSafeIMENameLengthForNTInTchars = 64;
+
+const char kCompanyNameInEnglish[] = "Google";
 const char kEventPathPrefix[] = "Global\\GoogleJapaneseInput.event.";
 const char kMutexPathPrefix[] = "Global\\GoogleJapaneseInput.mutex.";
-#else
-const char kEventPathPrefix[] = "Global\\Mozc.event.";
-const char kMutexPathPrefix[] = "Global\\Mozc.mutex.";
-#endif  // GOOGLE_JAPANESE_INPUT_BUILD
 const char kMozcServerName[] = "GoogleIMEJaConverter.exe";
-const char kIMEFile[] = "GoogleIMEJa.ime";
+const char kOldIMEFile[] = "GoogleIMEJa.ime";
+const char kIMEFile[] = "GIMEJa.ime";
 const char kMozcTIP32[] = "GoogleIMEJaTIP32.dll";
 const char kMozcTIP64[] = "GoogleIMEJaTIP64.dll";
+const char kMozcBroker32[] = "GoogleIMEJaBroker32.exe";
+const char kMozcBroker64[] = "GoogleIMEJaBroker64.exe";
 const char kMozcTool[] = "GoogleIMEJaTool.exe";
 const char kMozcRenderer[] = "GoogleIMEJaRenderer.exe";
 const char kCrashReportApp[] = "GoogleIMEJaCrashReporter.exe";
@@ -69,17 +72,19 @@ const wchar_t kCandidateWindowClassName[]
     = L"GoogleJapaneseInputCandidateWindow";
 const wchar_t kCompositionWindowClassName[]
     = L"GoogleJapaneseInputCompositionWindow";
-// IMM32 caches UIWnd class name (probably per window station).
+// This UIWnd class name has been used by the helper ime module which does some
+// workarounds against some problematic versions of msctf.dll on Windows XP.
+// Please note IMM32 caches UIWnd class name (probably per window station).
 // Whenever you rename it, restart Windows before testing.
 const wchar_t kDummyIMEUIWndClassName[] = L"GIMEJaUIWnd";
-// |kIMEUIWndClassName| is chosen to be the same to |kDummyIMEUIWndClassName|
-// because:
-//  1. They are exclusive.  If an installer contains a dummy ime, it does not
-//     contains a (real) ime.
-//  2. IMM32 caches UIWnd class name.  To avoid a reboot during the transition
-//     from a dummy ime to new (real) ime, their UIWnd class names should be
-//     equal to each other.
-const wchar_t kIMEUIWndClassName[] = L"GIMEJaUIWnd";
+// This UIWnd class name should be used by and only by the actual IMM32
+// version.  Make sure that |kIMEUIWndClassName| is different from
+// |kDummyIMEUIWndClassName| so that the dummy IME and the actual IME can
+// coexist in the same process.  Such a situation is not pleasant but may occur
+// in existing processes during silent / background upgrading.
+// Please note IMM32 caches UIWnd class name (probably per window station).
+// Whenever you rename it, restart Windows before testing.
+const wchar_t kIMEUIWndClassName[] = L"GIMEJaUIWindow";
 // UIWnd class name (including the null terminator) for the IMM32 can be up to
 // 16 TCHARs.
 const int kIMEUIwndClassNameLimitInTchars = 16;
@@ -91,6 +96,10 @@ const wchar_t kMozcRegKey[] = L"Software\\Google\\Google Japanese Input";
 const wchar_t kElevatedProcessDisabledKey[]
     = L"Software\\Policies\\Google\\Google Japanese Input\\Preferences";
 const wchar_t kDefaultKeyboardLayout[] = L"00000411";
+#else
+const char kEventPathPrefix[] = "Global\\Mozc.event.";
+const char kMutexPathPrefix[] = "Global\\Mozc.mutex.";
+#endif  // GOOGLE_JAPANESE_INPUT_BUILD
 #elif defined(OS_MACOSX)
 const char kMozcServerName[] = kProductPrefix "Converter.app";
 #ifdef GOOGLE_JAPANESE_INPUT_BUILD
@@ -103,6 +112,10 @@ const char kMozcServerName[] = "mozc_server";
 const char kEventPathPrefix[] = "mozc.event.";
 const char kMozcTool[] = "mozc_tool";
 #endif
+
+const char kWordRegisterEnvironmentName[] = "default_entry_of_word_register";
+const char kWordRegisterEnvironmentReadingName[] =
+    "default_reading_entry_of_word_register";
 }  // namespace mozc
 
 #endif  // MOZC_BASE_CONST_H_
