@@ -1,5 +1,6 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 2010, Google Inc.
+# Copyright 2010-2011, Google Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -89,14 +90,36 @@ def GetNumberOfProcessors():
   elif IsLinux():
     # Count the number of 'vendor_id' in /proc/cpuinfo, assuming that
     # each line corresponds to one logical CPU.
-    file = open('/proc/cpuinfo', 'r')
-    count = len([line for line in file if line.find('vendor_id') != -1])
-    file.close()
+    cpuinfo = open('/proc/cpuinfo', 'r')
+    count = len([line for line in cpuinfo if line.find('vendor_id') != -1])
+    cpuinfo.close()
     return count
   else:
     return 1
 
 
+def GetBuildBaseName(options):
+  """Returns the buildb ase directory."""
+  if options.build_base:
+    return options.build_base
+  # For some reason, xcodebuild does not accept absolute path names for
+  # the -project parameter. Convert the original_directory_name to a
+  # relative path from the build top level directory.
+  build_base = ''
+  if IsMac():
+    build_base = os.path.join(os.getcwd(), 'out_mac')
+  elif IsWindows():
+    build_base = os.path.join(os.getcwd(), 'out_win')
+  elif IsLinux():
+    # On Linux, seems there is no way to specify the build_base directory
+    # inside common.gypi
+    build_base = 'out_linux'
+  else:
+    logging.error('Unsupported platform: %s', os.name)
+
+  return build_base
+
+
 def GetGeneratorName():
   """Gets the generator name based on the platform."""
   generator = 'make'
@@ -129,6 +152,8 @@ def GenerateVersionFile(version_template_path, version_path):
     open(version_path, 'w').write(version_definition)
 
 
+
+
 def GetVersionFileNames(options):
   """Gets the (template of version file, version file) pair."""
   template_path = '%s/%s' % (SRC_DIR, options.version_file)
@@ -239,14 +264,13 @@ def CleanBuildFilesAndDirectories():
   file_names.append('third_party/rx/rx.gyp')
   # Collect stuff in the top-level directory.
   directory_names.append('mozc_build_tools')
-  if IsMac():
-    directory_names.append('out_mac')
-  elif IsLinux():
+
+  (options, unused_args) = ParseCleanOptions()
+  directory_names.append(GetBuildBaseName(options))
+  if IsLinux():
     file_names.append('Makefile')
-    directory_names.append('out_linux')
   elif IsWindows():
     file_names.append('third_party/breakpad/breakpad.gyp')
-    directory_names.append('out_win')
   # Remove files.
   for file_name in file_names:
     RemoveFile(file_name)
@@ -302,6 +326,11 @@ def GypMain(deps_file_name):
 
   # Get and show the list of .gyp file names.
   gyp_file_names = GetGypFileNames()
+  if not options.chewing:
+    # chewing/chewing.gyp is automatically included because of the
+    # policy.  We explicitly exclude it here.
+    gyp_file_names = [gyp_file for gyp_file in gyp_file_names
+                      if not gyp_file.endswith('chewing.gyp')]
   print 'GYP files:'
   for file_name in gyp_file_names:
     print '- %s' % file_name
@@ -329,14 +358,26 @@ def GypMain(deps_file_name):
   if options.coverage:
     command_line.extend(['-D', 'coverage=1'])
 
-  # Check the version and determine if the building version is
-  # dev-channel or not.
+  command_line.extend(['-D', 'build_base=%s' % GetBuildBaseName(options)])
+
+  # Check the version and determine if the building version is dev-channel or
+  # not. Note that if --channel_dev is explicitly set, we don't check the
+  # version number.
   (template_path, unused_version_path) = GetVersionFileNames(options)
   version = mozc_version.MozcVersion(template_path, expand_daily=False)
-  if options.channel_dev or version.IsDevChannel():
+  if options.channel_dev is None:
+    options.channel_dev = version.IsDevChannel()
+  if options.channel_dev:
     command_line.extend(['-D', 'channel_dev=1'])
 
 
+  # Dictionary configuration
+  if options.target_platform == 'ChromeOS':
+    # Note that the OSS version of mozc ignores the dictionary variable.
+    command_line.extend(['-D', 'dictionary=small'])
+  else:
+    command_line.extend(['-D', 'dictionary=desktop'])
+
   RunOrDie(command_line)
 
 
@@ -344,12 +385,15 @@ def GypMain(deps_file_name):
   print 'Done'
 
 
-def RunTests(configuration, calculate_coverage):
+def RunTests(configuration, unused_calculate_coverage):
   """Run built tests actually.
 
   Args:
     configuration: build configuration ('Release' or 'Debug')
-    calculate_coverage: True if runtests calculates the test coverage.
+    unused_calculate_coverage: True if runtests calculates the test coverage.
+
+  Raises:
+    RunOrDieError: One or more tests have failed.
   """
   # Currently we don't check calculate at all -- just run tests.
   if IsMac():
@@ -365,6 +409,10 @@ def RunTests(configuration, calculate_coverage):
   options = []
 
 
+  # Specify the log_dir directory.
+  # base_path looks like out_mac/Debug.
+  options.append('--log_dir=%s' % base_path)
+
   failed_tests = []
   # This is a silly algorithm: it runs *all* tests built in the target
   # directory.  Therefore, if you build multiple tests without
@@ -382,7 +430,7 @@ def RunTests(configuration, calculate_coverage):
     except RunOrDieError, e:
       print e
       failed_tests.append(binary)
-  if len(failed_tests) > 0:
+  if failed_tests:
     raise RunOrDieError('\n'.join(['following tests failed'] + failed_tests))
 
 
@@ -482,16 +530,67 @@ def ParseGypOptions():
   parser.add_option('--coverage', action='store_true', dest='coverage',
                     help='use code coverage analysis build options',
                     default=False)
-  parser.add_option('--channel_dev', action='store_true', dest='channel_dev',
-                    help='Specify this to build dev channel explicitly',
-                    default=False)
+  parser.add_option('--channel_dev', action='store', dest='channel_dev',
+                    type='int',
+                    help='Pass --channel_dev=1 if you need to build mozc with '
+                    'the CHANNEL_DEV macro enabled. Pass 0 if you don\'t need '
+                    'the macro. If --channel_dev= flag is not passed, Mozc '
+                    'version is used to deretmine if the macro is necessary.')
   parser.add_option('--version_file', dest='version_file',
                     help='use the specified version template file',
                     default='mozc_version_template.txt')
+  parser.add_option('--rsync', dest='rsync', default=False, action='store_true',
+                    help='use rsync to copy files instead of builtin function')
+
+  parser.add_option('--build_base', dest='build_base',
+                    help='specify the base directory of the built binaries.')
+
+  parser.add_option('--chewing', dest='chewing', action='store_true',
+                    default=False, help='include chewing gyp recipe.  '
+                    'This flag is false by default because it may require you '
+                    'to install libchewing in your environment.')
+
+
+  # Linux environment can build both for Linux and ChromeOS.
+  # This option enable this script to know which build (Linux or ChromeOS)
+  # should be done. If you want ChromeOS build, specify "ChromeOS".
+  parser.add_option('--target_platform', dest='target_platform',
+                    help='if you want ChromeOS build, specify "ChromeOS"')
+
   (options, unused_args) = parser.parse_args()
   return options
 
 
+def ParseMetaTarget(meta_target_name):
+  """Returns a list of build targets with expanding meta target name.
+
+  If the specified name is 'package', returns a list of build targets for
+  building production package.
+
+  If the specified name is not a meta target name, returns it as a list.
+
+  Args:
+    meta_target_name: meta target name to be expanded.
+
+  Returns:
+    A list of build targets with meta target names expanded.
+  """
+  if meta_target_name != 'package':
+    return [meta_target_name]
+
+  if IsLinux():
+    targets = ['%s/unix/ibus/ibus.gyp:ibus_mozc',
+               '%s/server/server.gyp:mozc_server',
+               '%s/gui/gui.gyp:mozc_tool']
+  elif IsMac():
+    targets = ['%s/mac/mac.gyp:DiskImage']
+  elif IsWindows():
+    targets = ['%s/win32/build32/build32.gyp:',
+               '%s/win32/build64/build64.gyp:',
+               '%s/win32/installer/installer.gyp:']
+  return [(target % SRC_DIR) for target in targets]
+
+
 def ParseBuildOptions():
   """Parse command line options for the build command."""
   parser = optparse.OptionParser(usage='Usage: %prog build [options]')
@@ -499,18 +598,18 @@ def ParseBuildOptions():
                     help='run jobs in parallel')
   parser.add_option('--configuration', '-c', dest='configuration',
                     default='Debug', help='specify the build configuration.')
-  parser.add_option('--build_base', dest='build_base',
-                    help='specify the base directory of the built binaries.')
   parser.add_option('--noqt', action='store_true', dest='noqt', default=False)
   parser.add_option('--version_file', dest='version_file',
                     help='use the specified version template file',
                     default='mozc_version_template.txt')
-  if IsWindows():
-    parser.add_option('--platform', '-p', dest='platform',
-                      default='Win32',
-                      help='specify the target plaform: [Win32|x64]')
+
+  # On Linux, seems there is no way to set build_base in the ParseGyp section
+  if IsLinux():
+    parser.add_option('--build_base', dest='build_base',
+                      help='specify the base directory of the built binaries.')
+
   # default Qt dir to support the current build procedure for Debian.
-  default_qtdir = '/usr/local/Trolltech/Qt-4.5.2'
+  default_qtdir = '/usr/local/Trolltech/Qt-4.6.3'
   if IsWindows():
     default_qtdir = None
   parser.add_option('--qtdir', dest='qtdir',
@@ -519,11 +618,15 @@ def ParseBuildOptions():
 
   (options, args) = parser.parse_args()
 
-  targets = args
-  if not targets:
+  original_targets = args
+  if not original_targets:
     PrintErrorAndExit('No build target is specified.')
 
-  return (options, args)
+  targets = []
+  for target in original_targets:
+    targets.extend(ParseMetaTarget(target))
+
+  return (options, targets)
 
 
 def ParseRuntestsOptions():
@@ -542,6 +645,19 @@ def ParseRuntestsOptions():
   return (options, args)
 
 
+def ParseCleanOptions():
+  """Parse command line options for the clean command."""
+  parser = optparse.OptionParser(
+      usage='Usage: %prog clean [-- build options]')
+
+  parser.add_option('--build_base', dest='build_base',
+                    help='specify the base directory of the built binaries.')
+
+  (options, args) = parser.parse_args()
+
+  return (options, args)
+
+
 def ParseTarget(target):
   """Parses the target string."""
   if not ':' in target:
@@ -609,14 +725,7 @@ def GetRelpath(path, start):
 
 def BuildOnMac(options, targets, original_directory_name):
   """Build the targets on Mac."""
-  # For some reason, xcodebuild does not accept absolute path names for
-  # the -project parameter. Convert the original_directory_name to a
-  # relative path from the build top level directory.
   original_directory_relpath = GetRelpath(original_directory_name, os.getcwd())
-  if options.build_base:
-    sym_root = options.build_base
-  else:
-    sym_root = os.path.join(os.getcwd(), 'out_mac')
   for target in targets:
     (gyp_file_name, target_name) = ParseTarget(target)
     gyp_file_name = os.path.join(original_directory_relpath, gyp_file_name)
@@ -627,7 +736,6 @@ def BuildOnMac(options, targets, original_directory_name):
               '-configuration', options.configuration,
               '-target', target_name,
               '-parallelizeTargets',
-              'SYMROOT=%s' % sym_root,
               'BUILD_WITH_GYP=1'])
 
 
@@ -644,6 +752,8 @@ def LocateVCBuildDir():
   PrintErrorAndExit('Failed to locate vcbuild.exe')
 
 
+
+
 def BuildOnWindows(options, targets, original_directory_name):
   """Build the target on Windows."""
   # TODO(yukawa): make a python module to set up environment for vcbuild.
@@ -657,31 +767,6 @@ def BuildOnWindows(options, targets, original_directory_name):
     os.environ['PATH'] = abs_vcbuild_dir
 
 
-  os.environ['INCLUDE'] = ''
-  os.environ['LIB'] = ''
-  os.environ['LIBPATH'] = ''
-
-  for target in targets:
-    # TODO(yukawa): target name is currently ignored.
-    (gyp_file_name, _) = ParseTarget(target)
-    gyp_file_name = os.path.join(original_directory_name, gyp_file_name)
-    CheckFileOrDie(gyp_file_name)
-    (sln_base_name, _) = os.path.splitext(gyp_file_name)
-    sln_file_path = os.path.abspath('%s.sln' % sln_base_name)
-    build_concurrency = 1
-    # To use different toolsets for vcbuild, we set %PATH%, %INCLUDE%, %LIB%,
-    # %LIBPATH% and specify /useenv option here.  See the following article
-    # for details.
-    # http://blogs.msdn.com/vcblog/archive/2007/12/30/using-different-toolsets-
-    #   for-vc-build.aspx
-    RunOrDie(['vcbuild',
-              '/useenv',  # Use %PATH%, %INCLUDE%, %LIB%, %LIBPATH%
-              '/M%d' % build_concurrency,  # Use concurrent build
-              '/time',    # Show build time
-              '/platform:%s' % options.platform,
-              sln_file_path,
-              '%s|%s' % (options.configuration, options.platform)])
-
 
 def BuildMain(original_directory_name):
   """The main function for the 'build' command."""
@@ -692,6 +777,7 @@ def BuildMain(original_directory_name):
   (template_path, version_path) = GetVersionFileNames(options)
   GenerateVersionFile(template_path, version_path)
 
+
   if not options.noqt:
     # Set $QTDIR for mozc_tool
     if options.qtdir:
