@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@
 #include "base/file_stream.h"
 #include "base/process.h"
 #include "base/run_level.h"
+#include "base/singleton.h"
 #include "base/util.h"
 #include "base/version.h"
 #include "ipc/ipc.h"
@@ -48,6 +49,10 @@
 #include "base/mac_process.h"
 #endif
 
+#ifdef OS_WINDOWS
+#include <windows.h>
+#endif
+
 namespace mozc {
 namespace client {
 
@@ -244,15 +249,14 @@ void Session::PushHistory(const commands::Input &input,
 // Clear the history and push IMEOn command for initialize session.
 void Session::ResetHistory() {
   history_inputs_.clear();
-#ifdef OS_WINDOWS
-  // On Windows, we should send ON key at the first of each input session
+#if defined(OS_WINDOWS) || defined(OS_MACOSX)
+  // On Windows/Mac, we should send ON key at the first of each input session
   // excepting the very first session, because when the session is restored,
   // its state is direct. On the first session, users should send ON key
   // by themselves.
   // Note that we are assuming that ResetHistory is called only when the
   // client is ON.
   // TODO(toshiyuki): Make sure that this assuming is reasonable or not.
-  // TODO(toshiyuki): Investigate for Mac and remove #ifdef guard.
   if (last_mode_ != commands::DIRECT) {
     commands::Input input;
     input.set_type(commands::Input::SEND_KEY);
@@ -394,11 +398,28 @@ void Session::set_server_program(const string &program_path) {
   server_launcher_->set_server_program(program_path);
 }
 
+void Session::set_client_capability(const commands::Capability &capability) {
+  client_capability_.CopyFrom(capability);
+}
+
 bool Session::CreateSession() {
   id_ = 0;
   commands::Input input;
   input.set_type(commands::Input::CREATE_SESSION);
 
+  input.mutable_capability()->CopyFrom(client_capability_);
+
+  commands::ApplicationInfo *info = input.mutable_application_info();
+  DCHECK(info);
+
+#ifdef OS_WINDOWS
+  info->set_process_id(static_cast<uint32>(::GetCurrentProcessId()));
+  info->set_thread_id(static_cast<uint32>(::GetCurrentThreadId()));
+#else
+  info->set_process_id(static_cast<uint32>(getpid()));
+  info->set_thread_id(0);
+#endif
+
   commands::Output output;
   if (!CheckVersionOrRestartServerInternal(input, &output)) {
     LOG(ERROR) << "CheckVersionOrRestartServer() failed";
@@ -864,5 +885,28 @@ bool Session::OpenBrowser(const string &url) {
   return true;
 }
 
+namespace {
+class DefaultSessionFactory : public SessionFactoryInterface {
+ public:
+  virtual SessionInterface *NewSession() {
+    return new Session;
+  }
+};
+
+SessionFactoryInterface *g_session_factory = NULL;
+}  // namespace
+
+SessionInterface *SessionFactory::NewSession() {
+  if (g_session_factory == NULL) {
+    return Singleton<DefaultSessionFactory>::get()->NewSession();
+  } else {
+    return g_session_factory->NewSession();
+  }
+}
+
+void SessionFactory::SetSessionFactory(
+    SessionFactoryInterface *session_factory) {
+  g_session_factory = session_factory;
+}
 }  // namespace client
 }  // namespace mozc
