@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,11 +32,13 @@
 #include "composer/composer.h"
 
 #include "base/util.h"
-#include "converter/character_form_manager.h"
+#include "base/singleton.h"
 #include "composer/internal/composition.h"
+#include "composer/internal/composition_input.h"
 #include "composer/internal/mode_switching_handler.h"
 #include "composer/internal/transliterators_ja.h"
 #include "composer/table.h"
+#include "converter/character_form_manager.h"
 #include "session/commands.pb.h"
 #include "session/config_handler.h"
 #include "session/config.pb.h"
@@ -187,14 +189,16 @@ transliteration::TransliterationType GetTransliterationTypeFromCompositionMode(
 }  // namespace
 
 static const size_t kMaxPreeditLength = 256;
-Composer::Composer(CompositionInterface *composition)
+Composer::Composer()
     : position_(0),
+      is_new_input_(true),
       input_mode_(transliteration::HIRAGANA),
       output_mode_(transliteration::HIRAGANA),
       comeback_input_mode_(transliteration::HIRAGANA),
       capital_sequence_count_(0),
+      composition_(new Composition),
       max_length_(kMaxPreeditLength) {
-  composition_.reset(composition);
+  composition_->SetTable(Singleton<Table>::get());
   SetInputMode(transliteration::HIRAGANA);
   Reset();
 }
@@ -203,6 +207,10 @@ Composer::~Composer() {}
 
 void Composer::Reset() {
   EditErase();
+  ResetInputMode();
+}
+
+void Composer::ResetInputMode() {
   SetInputMode(comeback_input_mode_);
 }
 
@@ -222,6 +230,7 @@ void Composer::SetInputMode(transliteration::TransliterationType mode) {
   comeback_input_mode_ = mode;
   input_mode_ = mode;
   capital_sequence_count_ = 0;
+  is_new_input_ = true;
   composition_->SetInputMode(GetTransliterator(mode));
 }
 
@@ -231,6 +240,7 @@ void Composer::SetTemporaryInputMode(
   comeback_input_mode_ = input_mode_;
   input_mode_ = mode;
   capital_sequence_count_ = 0;
+  is_new_input_ = true;
   composition_->SetInputMode(GetTransliterator(mode));
 }
 
@@ -241,20 +251,17 @@ void Composer::UpdateInputMode() {
     return;
   }
 
-  if (position_ == composition_->GetLength()) {
-    // Do nothing, if the cursor is in the end of the composition.
-    return;
-  }
-
-  const TransliteratorInterface *lhs_t12r =
+  const TransliteratorInterface *current_t12r =
       composition_->GetTransliterator(position_);
-  const TransliteratorInterface *rhs_t12r =
-      composition_->GetTransliterator(position_ + 1);
-  if (lhs_t12r == rhs_t12r) {
-    // If the current cursor is between the same character type like
-    // "A|B" and "あ|い", the input mode follows the character type.
-    input_mode_ = GetTransliterationType(lhs_t12r, comeback_input_mode_);
+  if (position_ == composition_->GetLength() ||
+      current_t12r == composition_->GetTransliterator(position_ + 1)) {
+    // - The cursor is at the tail of composition.
+    //   Use last character's transliterator as the input mode.
+    // - If the current cursor is between the same character type like
+    //   "A|B" and "あ|い", the input mode follows the character type.
+    input_mode_ = GetTransliterationType(current_t12r, comeback_input_mode_);
     capital_sequence_count_ = 0;
+    is_new_input_ = true;
     composition_->SetInputMode(GetTransliterator(input_mode_));
     return;
   }
@@ -284,21 +291,18 @@ void Composer::SetOutputMode(transliteration::TransliterationType mode) {
   output_mode_ = mode;
   composition_->SetTransliterator(
       0, composition_->GetLength(), GetTransliterator(mode));
-  MoveCursorToEnd();
+  position_ = composition_->GetLength();
 }
 
-void Composer::InsertCharacterAt(size_t pos, const string &input) {
+void Composer::InsertCharacter(const string &key) {
   if (!EnableInsert()) {
     return;
   }
-  position_ = composition_->InsertAt(pos, input);
-}
-
-void Composer::InsertCharacter(const string &input) {
-  if (!EnableInsert()) {
-    return;
-  }
-  position_ = composition_->InsertAt(position_, input);
+  CompositionInput input;
+  input.set_raw(key);
+  input.set_is_new_input(is_new_input_);
+  position_ = composition_->InsertInput(position_, input);
+  is_new_input_ = false;
 }
 
 void Composer::InsertCharacterPreedit(const string &input) {
@@ -306,11 +310,48 @@ void Composer::InsertCharacterPreedit(const string &input) {
 }
 
 void Composer::InsertCharacterKeyAndPreedit(const string &key,
-                                       const string &preedit) {
+                                            const string &preedit) {
   if (!EnableInsert()) {
     return;
   }
-  position_ = composition_->InsertKeyAndPreeditAt(position_, key, preedit);
+  CompositionInput input;
+  input.set_raw(key);
+  input.set_conversion(preedit);
+  input.set_is_new_input(is_new_input_);
+  position_ = composition_->InsertInput(position_, input);
+  is_new_input_ = false;
+}
+
+void Composer::InsertCharacterPreeditAt(size_t pos, const string &input) {
+  InsertCharacterKeyAndPreeditAt(pos, input, input);
+}
+
+void Composer::InsertCharacterKeyAndPreeditAt(size_t pos,
+                                              const string &key,
+                                              const string &preedit) {
+  const size_t position_before_insertion = position_;
+  const size_t length_before_insertion = composition_->GetLength();
+  const size_t insertion_length = Util::CharsLen(preedit);
+
+  composition_->SetInputMode(
+      Transliterators::GetConversionStringSelector());
+
+  CompositionInput input;
+  input.set_raw(key);
+  input.set_conversion(preedit);
+  input.set_is_new_input(true);
+  composition_->InsertInput(pos, input);
+
+  DCHECK_EQ(insertion_length,
+            composition_->GetLength() - length_before_insertion);
+
+  composition_->SetInputMode(GetTransliterator(input_mode_));
+
+  position_ = position_before_insertion;
+  if (position_before_insertion >= pos) {
+    position_ += insertion_length;
+  }
+  is_new_input_ = false;
 }
 
 bool Composer::InsertCharacterKeyEvent(const commands::KeyEvent &key) {
@@ -364,6 +405,7 @@ bool Composer::InsertCharacterKeyEvent(const commands::KeyEvent &key) {
     // Romaji input usually does not has key_string.  Note that, the
     // existence of key_string never determine if the input mode is
     // Kana or Romaji.
+
     if ('A' <= key_char && key_char <= 'Z') {
       if (GET_CONFIG(shift_key_mode_switch) ==
           config::Config::ASCII_INPUT_MODE) {
@@ -375,17 +417,17 @@ bool Composer::InsertCharacterKeyEvent(const commands::KeyEvent &key) {
         }
       } else if (GET_CONFIG(shift_key_mode_switch) ==
                  config::Config::KATAKANA_INPUT_MODE) {
-        if (input_mode_ == transliteration::FULL_KATAKANA) {
-          // Do nothing.
-        } else {
+        if (input_mode_ == transliteration::HIRAGANA) {
           SetTemporaryInputMode(transliteration::FULL_KATAKANA);
+        } else {
+          // Do nothing.
         }
       }
       ++capital_sequence_count_;
-    }
-    // When capital input continues, the next lower input is the end
-    // of temporary half-width Ascii input.
-    if ('a' <= key_char && key_char <= 'z') {
+    } else if ('a' <= key_char && key_char <= 'z') {
+      // When capital input continues, the next lower input is the end
+      // of temporary half-width Ascii input.
+
       if (capital_sequence_count_ > 1 &&
           GET_CONFIG(shift_key_mode_switch) ==
           config::Config::ASCII_INPUT_MODE) {
@@ -396,6 +438,11 @@ bool Composer::InsertCharacterKeyEvent(const commands::KeyEvent &key) {
         SetInputMode(comeback_input_mode_);
       }
       capital_sequence_count_ = 0;
+    } else {
+      // If the key_char is not an alphabet, reset capital_sequence_count_
+      // because "Continuous capital input" feature should be reset
+      // when the input meets non-alphabet character.
+      capital_sequence_count_ = 0;
     }
     InsertCharacter(input);
   }
@@ -419,49 +466,55 @@ void Composer::Delete() {
 void Composer::EditErase() {
   composition_->Erase();
   position_ = 0;
-  UpdateInputMode();
-}
-
-void Composer::BackspaceAt(size_t pos) {
-  if (pos == 0) {
-    return;
-  }
-  position_ = composition_->DeleteAt(pos - 1);
-  UpdateInputMode();
+  SetInputMode(comeback_input_mode_);
 }
 
 void Composer::Backspace() {
-  BackspaceAt(position_);
+  if (position_ == 0) {
+    return;
+  }
+
+  // In the view point of updating input mode,
+  // backspace is special case because new input mode is based on both
+  // new current character and *character to be deleted*.
+
+  // At first, move to left.
+  // Now the cursor is between 'new current character'
+  // and 'character to be deleted'.
+  --position_;
+
+  // Update input mode based on both 'new current character' and
+  // 'character to be deleted'.
   UpdateInputMode();
+
+  // Delete 'character to be deleted'
+  position_ = composition_->DeleteAt(position_);
 }
 
 void Composer::MoveCursorLeft() {
   if (position_ > 0) {
     --position_;
-    UpdateInputMode();
   }
+  UpdateInputMode();
 }
 
 void Composer::MoveCursorRight() {
   if (position_ < composition_->GetLength()) {
     ++position_;
-    UpdateInputMode();
   }
+  UpdateInputMode();
 }
 
 void Composer::MoveCursorToBeginning() {
-  if (position_ != 0) {
-    position_ = 0;
-    UpdateInputMode();
-  }
+  position_ = 0;
+  SetInputMode(comeback_input_mode_);
 }
 
 void Composer::MoveCursorToEnd() {
-  const size_t cursor_end = composition_->GetLength();
-  if (position_ != cursor_end) {
-    position_ = cursor_end;
-    UpdateInputMode();
-  }
+  position_ = composition_->GetLength();
+  // Behavior between MoveCursorToEnd and MoveCursorToRight is different.
+  // MoveCursorToEnd always makes current input mode default.
+  SetInputMode(comeback_input_mode_);
 }
 
 void Composer::GetPreedit(string *left, string *focused, string *right) const {
@@ -723,23 +776,24 @@ void Composer::AutoSwitchMode() {
   }
 }
 
-// static
-Composer *Composer::Create(const Table *table) {
-  Composition *composition = new Composition;
-  composition->SetTable(table);
-  Composer *composer = new Composer(composition);
-  return composer;
+bool Composer::ShouldCommit() const {
+  return composition_->ShouldCommit();
 }
 
 namespace {
 enum Script {
-  ALPHANUM,     // 0 - 9, "０" - "９", alphabet characters.
+  ALPHABET,   // alphabet characters or symbols
+  NUMBER,     // 0 - 9, "０" - "９"
   JA_HYPHEN,  // "ー"
   JA_COMMA,   // "、"
   JA_PERIOD,  // "。"
   OTHERS,
 };
 
+bool IsAlphabetOrNumber(const Script script) {
+  return (script == ALPHABET) || (script == NUMBER);
+}
+
 static const char *kNumberSymbols[] = {
   "+", "*", "/", "=", "(", ")", "<", ">",
   // "＋", "＊", "／", "＝",
@@ -762,7 +816,7 @@ bool Composer::TransformCharactersForNumbers(string *query) {
   vector<Script> char_scripts;
   // flags to determine whether continue to the next step.
   bool has_symbols = false;
-  bool has_numbers = false;
+  bool has_alphanumerics = false;
   for (size_t i = 0; i < chars_len; ++i) {
     Script script = OTHERS;
     const string one_char = Util::SubString(*query, i, 1);
@@ -776,15 +830,15 @@ bool Composer::TransformCharactersForNumbers(string *query) {
       has_symbols = true;
       script = JA_PERIOD;
     } else if (Util::IsScriptType(one_char, Util::NUMBER)) {
-      has_numbers = true;
-      script = ALPHANUM;
+      has_alphanumerics = true;
+      script = NUMBER;
     } else if (Util::IsScriptType(one_char, Util::ALPHABET)) {
-      has_numbers = true;
-      script = ALPHANUM;
+      has_alphanumerics = true;
+      script = ALPHABET;
     } else {
       for (size_t j = 0; j < arraysize(kNumberSymbols); ++j) {
         if (one_char == kNumberSymbols[j]) {
-          script = ALPHANUM;
+          script = ALPHABET;
           break;
         }
       }
@@ -792,8 +846,8 @@ bool Composer::TransformCharactersForNumbers(string *query) {
     char_scripts.push_back(script);
   }
   DCHECK_EQ(chars_len, char_scripts.size());
-  if (!has_numbers || !has_symbols) {
-    VLOG(1) << "The query does not contain number, symbol or neither.";
+  if (!has_alphanumerics || !has_symbols) {
+    VLOG(1) << "The query contains neither alphanumeric nor symbol.";
     return false;
   }
 
@@ -801,40 +855,27 @@ bool Composer::TransformCharactersForNumbers(string *query) {
   bool transformed = false;
   for (size_t i = 0; i < chars_len; ++i) {
     const Script script = char_scripts[i];
-    if (script == OTHERS || script == ALPHANUM) {
+    if (script == OTHERS || IsAlphabetOrNumber(script)) {
       // Append one character.
       Util::SubString(*query, i, 1, &transformed_query);
       continue;
     }
 
-    // "ー" should be "−" if the previous character is a digit, or the
-    // previous character is empty and the next character is a digit.
+    // JA_HYPHEN(s) "ー" is/are transformed to "−" if:
+    // (i) query has one and only one leading JA_HYPHEN followed by a number,
+    // (ii) JA_HYPHEN(s) follow(s) after an alphanumeric (ex. 0-, 0----, etc).
+    // Note that rule (i) implies that if query starts with more than
+    // one JA_HYPHENs, those JA_HYPHENs are not transformed.
     if (script == JA_HYPHEN) {
       bool check = false;
-      // If there is no previous char, the next character decides the
-      // transform.
-      if (i == 0) {
-        // Next char should exist and be a number.
-        if (i+1 < chars_len && char_scripts[i+1] == ALPHANUM) {
-          check = true;
-        } else if (i+2 < chars_len &&
-                   char_scripts[i+1] == JA_HYPHEN &&
-                   char_scripts[i+2] == ALPHANUM) {
-          // Otherwise, the next character should also be JA_HYPHEN
-          // and the i+2 th characters should be an alphanumeric (eg. 1--2).
-          check = true;
-        }
+      if (i == 0 && chars_len > 1) {
+        check = (char_scripts[1] == NUMBER);
       } else {
-        // If previous character(s) is an Alphanumeric or sequence of
-        // JA_HYPHEN terminated with an Alphanumeric, the target
-        // JA_HYPHEN should be transformed (ex. 0-, 0-----, etc).
         for (size_t j = i; j > 0; --j) {
           if (char_scripts[j - 1] == JA_HYPHEN) {
             continue;
           }
-          if (char_scripts[j - 1] == ALPHANUM) {
-            check = true;
-          }
+          check = IsAlphabetOrNumber(char_scripts[j - 1]);
           break;
         }
       }
@@ -854,14 +895,12 @@ bool Composer::TransformCharactersForNumbers(string *query) {
     }
 
     // "、" should be "，" if the previous character and the next
-    // character are both digits.
+    // character are both alphanumerics.
     if (script == JA_COMMA) {
       // Previous char should exist and be a number.
-      const bool lhs_check = (i > 0 && char_scripts[i-1] == ALPHANUM);
-      // Next three characters should exist and be all digits.
-      const bool rhs_check = (i+1 < chars_len && char_scripts[i+1] == ALPHANUM);
+      const bool lhs_check = (i > 0 && IsAlphabetOrNumber(char_scripts[i - 1]));
       // JA_COMMA should be transformed to COMMA.
-      if (lhs_check && rhs_check) {
+      if (lhs_check) {
         string append_char;
         CharacterFormManager::GetCharacterFormManager()->
             ConvertPreeditString("\xEF\xBC\x8C", &append_char);  // "，"
@@ -875,15 +914,12 @@ bool Composer::TransformCharactersForNumbers(string *query) {
     }
 
     // "。" should be "．" if the previous character and the next
-    // character are both digits.
+    // character are both alphanumerics.
     if (script == JA_PERIOD) {
       // Previous char should exist and be a number.
-      const bool lhs_check = (i > 0 && char_scripts[i-1] == ALPHANUM);
-      // Next three characters should exist and be a digit.
-      const bool rhs_check = (i+1 < chars_len && char_scripts[i+1] == ALPHANUM);
-
+      const bool lhs_check = (i > 0 && IsAlphabetOrNumber(char_scripts[i - 1]));
       // JA_PRERIOD should be transformed to PRERIOD.
-      if (lhs_check && rhs_check) {
+      if (lhs_check) {
         string append_char;
         CharacterFormManager::GetCharacterFormManager()->
             ConvertPreeditString("\xEF\xBC\x8E", &append_char);  // "．"
