@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -44,6 +44,7 @@
 #include "base/stats_config_util.h"
 #include "gui/config_dialog/keymap_editor.h"
 #include "gui/config_dialog/roman_table_editor.h"
+#include "gui/base/win_util.h"
 #include "ipc/ipc.h"
 #include "session/commands.pb.h"
 #include "session/config_handler.h"
@@ -60,7 +61,8 @@ namespace gui {
 
 ConfigDialog::ConfigDialog()
     : client_(new client::Session),
-      initial_preedit_method_(0) {
+      initial_preedit_method_(0),
+      initial_use_keyboard_to_change_preedit_method_(false) {
   setupUi(this);
   setWindowFlags(Qt::WindowSystemMenuHint);
   setWindowModality(Qt::NonModal);
@@ -116,6 +118,12 @@ ConfigDialog::ConfigDialog()
 
   inputModeComboBox->addItem(tr("Romaji"));
   inputModeComboBox->addItem(tr("Kana"));
+#ifdef OS_WINDOWS
+  // These options changing the preedit method by a hot key are only
+  // supported by Windows.
+  inputModeComboBox->addItem(tr("Romaji (switchable)"));
+  inputModeComboBox->addItem(tr("Kana (switchable)"));
+#endif  // OS_WINDOWS
 
   spaceCharacterFormComboBox->addItem(tr("Follow input mode"));
   spaceCharacterFormComboBox->addItem(tr("Fullwidth"));
@@ -142,6 +150,16 @@ ConfigDialog::ConfigDialog()
   verboseLevelComboBox->addItem(tr("1"));
   verboseLevelComboBox->addItem(tr("2"));
 
+  yenSignComboBox->addItem(tr("Yen Sign \xC2\xA5"));
+  yenSignComboBox->addItem(tr("Backslash \\"));
+
+#ifndef OS_MACOSX
+  // On Windows/Linux, yenSignCombBox can be hidden.
+  yenSignLabel->hide();
+  yenSignComboBox->hide();
+  // On Windows/Linux, useJapaneseLayout checkbox should be invisible.
+  useJapaneseLayout->hide();
+#endif  // !OS_MACOSX
 
   // signal/slot
   QObject::connect(configDialogButtonBox,
@@ -180,11 +198,15 @@ ConfigDialog::ConfigDialog()
                    SIGNAL(currentIndexChanged(int)),
                    this,
                    SLOT(SelectInputModeSetting(int)));
-
   QObject::connect(keymapSettingComboBox,
                    SIGNAL(currentIndexChanged(int)),
                    this,
                    SLOT(SelectKeymapSetting(int)));
+  QObject::connect(useAutoConversion,
+                   SIGNAL(stateChanged(int)),
+                   this,
+                   SLOT(SelectAutoConversionSetting(int)));
+
   QObject::connect(launchAdministrationDialogButton,
                    SIGNAL(clicked()),
                    this,
@@ -259,11 +281,19 @@ ConfigDialog::ConfigDialog()
   }
   ConvertFromProto(config);
 
+  SelectAutoConversionSetting(static_cast<int>(config.use_auto_conversion()));
+
   initial_preedit_method_ = static_cast<int>(config.preedit_method());
+  initial_use_keyboard_to_change_preedit_method_ =
+      config.use_keyboard_to_change_preedit_method();
 
   // If the keymap is a custome keymap (= 0), the buttion is activated.
   editKeymapButton->setEnabled(keymapSettingComboBox->currentIndex() == 0);
 
+#ifdef OS_WINDOWS
+  IMEHotKeyDisabledCheckBox->setChecked(WinUtil::GetIMEHotKeyDisabled());
+#endif
+
 #ifdef CHANNEL_DEV
   usageStatsCheckBox->setEnabled(false);
 #endif  // CHANNEL_DEV
@@ -273,12 +303,10 @@ ConfigDialog::~ConfigDialog() {
 }
 
 bool ConfigDialog::SetConfig(const config::Config &config) {
-#ifndef OS_MACOSX
   if (!client_->CheckVersionOrRestartServer()) {
     LOG(ERROR) << "CheckVersionOrRestartServer failed";
     return false;
   }
-#endif  // OS_MACOSX
 
   if (!client_->SetConfig(config)) {
     LOG(ERROR) << "SetConfig failed";
@@ -289,12 +317,10 @@ bool ConfigDialog::SetConfig(const config::Config &config) {
 }
 
 bool ConfigDialog::GetConfig(config::Config *config) {
-#ifndef OS_MACOSX
   if (!client_->CheckVersionOrRestartServer()) {
     LOG(ERROR) << "CheckVersionOrRestartServer failed";
     return false;
   }
-#endif  // OS_MACOSX
 
   if (!client_->GetConfig(config)) {
     LOG(ERROR) << "GetConfig failed";
@@ -322,7 +348,9 @@ bool ConfigDialog::Update() {
 
 #if defined(OS_WINDOWS) || defined(OS_LINUX)
   if (initial_preedit_method_ !=
-      static_cast<int>(config.preedit_method())) {
+      static_cast<int>(config.preedit_method()) ||
+      initial_use_keyboard_to_change_preedit_method_ !=
+      static_cast<int>(config.use_keyboard_to_change_preedit_method())) {
     QMessageBox::information(this,
                              tr("Mozc settings"),
                              tr("Romaji/Kana setting is enabled from"
@@ -336,12 +364,21 @@ bool ConfigDialog::Update() {
                           tr("Failed to update config"));
   }
 
+#ifdef OS_WINDOWS
+  if (!WinUtil::SetIMEHotKeyDisabled(IMEHotKeyDisabledCheckBox->isChecked())) {
+    // Do not show any dialog here, since this operation will not fail
+    // in almost all cases.
+    // TODO(taku): better to show dialog?
+    LOG(ERROR) << "Failed to update IME HotKey status";
+  }
+#endif
+
   return true;
 }
 
 void ConfigDialog::SetSendStatsCheckBox() {
   // On windows, usage_stats flag is managed by
-  // administration_dialog. http://b/issue?id=2889759
+  // administration_dialog. http://b/2889759
 #ifndef OS_WINDOWS
   const bool val = StatsConfigUtil::IsEnabled();
   usageStatsCheckBox->setChecked(val);
@@ -350,7 +387,7 @@ void ConfigDialog::SetSendStatsCheckBox() {
 
 void ConfigDialog::GetSendStatsCheckBox() const {
   // On windows, usage_stats flag is managed by
-  // administration_dialog. http://b/issue?id=2889759
+  // administration_dialog. http://b/2889759
 #ifndef OS_WINDOWS
   const bool val = usageStatsCheckBox->isChecked();
   StatsConfigUtil::SetEnabled(val);
@@ -374,12 +411,45 @@ do {  \
 #define GET_CHECKBOX(checkbox, field) \
 do { config->set_##field((checkbox)->isChecked());  } while (0)
 
+
+namespace {
+static const int kPreeditMethodSize = 2;
+
+void SetComboboxForPreeditMethod(const config::Config &config,
+                                 QComboBox *combobox) {
+  int index = static_cast<int>(config.preedit_method());
+#ifdef OS_WINDOWS
+  if (config.use_keyboard_to_change_preedit_method()) {
+    index += kPreeditMethodSize;
+  }
+#endif  // OS_WINDOWS
+  combobox->setCurrentIndex(index);
+}
+
+void GetComboboxForPreeditMethod(const QComboBox *combobox,
+                                 config::Config *config) {
+  int index = combobox->currentIndex();
+  if (index >= kPreeditMethodSize) {
+    // |use_keyboard_to_change_preedit_method| should be true and
+    // |index| should be adjusted to smaller than kPreeditMethodSize.
+    config->set_preedit_method(
+        static_cast<config::Config_PreeditMethod>(index - kPreeditMethodSize));
+    config->set_use_keyboard_to_change_preedit_method(true);
+  } else {
+    config->set_preedit_method(
+        static_cast<config::Config_PreeditMethod>(index));
+    config->set_use_keyboard_to_change_preedit_method(false);
+  }
+}
+}  // anonymous namespace
+
+
 // TODO(taku)
 // Actually ConvertFromProto and ConvertToProto are almost the same.
 // The difference only SET_ and GET_. We would like to unify the twos.
 void ConfigDialog::ConvertFromProto(const config::Config &config) {
-  // tab 1
-  SET_COMBOBOX(inputModeComboBox, PreeditMethod, preedit_method);
+  // tab1
+  SetComboboxForPreeditMethod(config, inputModeComboBox);
   SET_COMBOBOX(punctuationsSettingComboBox, PunctuationMethod,
                punctuation_method);
   SET_COMBOBOX(symbolsSettingComboBox, SymbolMethod, symbol_method);
@@ -399,18 +469,38 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
                history_learning_level);
   SET_CHECKBOX(singleKanjiDictionaryCheckBox, use_single_kanji_conversion);
   SET_CHECKBOX(symbolDictionaryCheckBox, use_symbol_conversion);
+  SET_CHECKBOX(emoticonDictionaryCheckBox, use_emoticon_conversion);
   SET_CHECKBOX(dateConversionCheckBox, use_date_conversion);
   SET_CHECKBOX(numberConversionCheckBox, use_number_conversion);
+  SET_CHECKBOX(calculatorCheckBox, use_calculator);
 
   // tab3
   SET_CHECKBOX(useAutoImeTurnOff, use_auto_ime_turn_off);
+
+  SET_CHECKBOX(useAutoConversion, use_auto_conversion);
+  kutenCheckBox->setChecked(
+      config.auto_conversion_key() &
+      config::Config::AUTO_CONVERSION_KUTEN);
+  toutenCheckBox->setChecked(
+      config.auto_conversion_key() &
+      config::Config::AUTO_CONVERSION_TOUTEN);
+  questionMarkCheckBox->setChecked(
+      config.auto_conversion_key() &
+      config::Config::AUTO_CONVERSION_QUESTION_MARK);
+  exclamationMarkCheckBox->setChecked(
+      config.auto_conversion_key() &
+      config::Config::AUTO_CONVERSION_EXCLAMATION_MARK);
+
   SET_COMBOBOX(shiftKeyModeSwitchComboBox,
                ShiftKeyModeSwitch,
                shift_key_mode_switch);
 
+  SET_CHECKBOX(useJapaneseLayout, use_japanese_layout);
+
   // tab4
   SET_CHECKBOX(historySuggestCheckBox, use_history_suggest);
   SET_CHECKBOX(dictionarySuggestCheckBox, use_dictionary_suggest);
+  SET_CHECKBOX(realtimeConversionCheckBox, use_realtime_conversion);
 
   suggestionsSizeSpinBox->setValue
       (max(1, min(9, static_cast<int>(config.suggestions_size()))));
@@ -422,14 +512,14 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
   // tab6
   SET_COMBOBOX(verboseLevelComboBox, int, verbose_level);
   SET_CHECKBOX(checkDefaultCheckBox, check_default);
+  SET_COMBOBOX(yenSignComboBox, YenSignCharacter, yen_sign_character);
 
   characterFormEditor->Load(config);
 }
 
 void ConfigDialog::ConvertToProto(config::Config *config) const {
-  // tab 1
-  GET_COMBOBOX(inputModeComboBox, PreeditMethod,
-               preedit_method);
+  // tab1
+  GetComboboxForPreeditMethod(inputModeComboBox, config);
   GET_COMBOBOX(punctuationsSettingComboBox, PunctuationMethod,
                punctuation_method);
   GET_COMBOBOX(symbolsSettingComboBox, SymbolMethod, symbol_method);
@@ -453,11 +543,37 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
                history_learning_level);
   GET_CHECKBOX(singleKanjiDictionaryCheckBox, use_single_kanji_conversion);
   GET_CHECKBOX(symbolDictionaryCheckBox, use_symbol_conversion);
+  GET_CHECKBOX(emoticonDictionaryCheckBox, use_emoticon_conversion);
   GET_CHECKBOX(dateConversionCheckBox, use_date_conversion);
   GET_CHECKBOX(numberConversionCheckBox, use_number_conversion);
+  GET_CHECKBOX(calculatorCheckBox, use_calculator);
 
   // tab3
   GET_CHECKBOX(useAutoImeTurnOff, use_auto_ime_turn_off);
+
+  GET_CHECKBOX(useAutoConversion, use_auto_conversion);
+
+  GET_CHECKBOX(useJapaneseLayout, use_japanese_layout);
+
+  uint32 auto_conversion_key = 0;
+  if (kutenCheckBox->isChecked()) {
+    auto_conversion_key |=
+        config::Config::AUTO_CONVERSION_KUTEN;
+  }
+  if (toutenCheckBox->isChecked()) {
+    auto_conversion_key |=
+        config::Config::AUTO_CONVERSION_TOUTEN;
+  }
+  if (questionMarkCheckBox->isChecked()) {
+    auto_conversion_key |=
+        config::Config::AUTO_CONVERSION_QUESTION_MARK;
+  }
+  if (exclamationMarkCheckBox->isChecked()) {
+    auto_conversion_key |=
+        config::Config::AUTO_CONVERSION_EXCLAMATION_MARK;
+  }
+  config->set_auto_conversion_key(auto_conversion_key);
+
   GET_COMBOBOX(shiftKeyModeSwitchComboBox,
                ShiftKeyModeSwitch,
                shift_key_mode_switch);
@@ -465,6 +581,7 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
   // tab4
   GET_CHECKBOX(historySuggestCheckBox, use_history_suggest);
   GET_CHECKBOX(dictionarySuggestCheckBox, use_dictionary_suggest);
+  GET_CHECKBOX(realtimeConversionCheckBox, use_realtime_conversion);
 
   config->set_suggestions_size
       (static_cast<uint32>(suggestionsSizeSpinBox->value()));
@@ -476,6 +593,7 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
   // tab6
   config->set_verbose_level(verboseLevelComboBox->currentIndex());
   GET_CHECKBOX(checkDefaultCheckBox, check_default);
+  GET_COMBOBOX(yenSignComboBox, YenSignCharacter, yen_sign_character);
 
   characterFormEditor->Save(config);
 }
@@ -508,15 +626,15 @@ void ConfigDialog::ClearUserHistory() {
       QMessageBox::question(
           this,
           tr("Mozc settings"),
-          tr("Do you want to clear personalization data?"),
+          tr("Do you want to clear personalization data? "
+             "Input history is not reset with this operation. "
+             "Please open \"suggestion\" tab to remove input history data."),
           QMessageBox::Ok | QMessageBox::Cancel,
           QMessageBox::Cancel)) {
     return;
   }
 
-#ifndef OS_MACOSX
   client_->CheckVersionOrRestartServer();
-#endif  // OS_MACOSX
 
   if (!client_->ClearUserHistory()) {
     QMessageBox::critical(
@@ -538,9 +656,7 @@ void ConfigDialog::ClearUserPrediction() {
     return;
   }
 
-#ifndef OS_MACOSX
   client_->CheckVersionOrRestartServer();
-#endif  // OS_MACOSX
 
   if (!client_->ClearUserPrediction()) {
     QMessageBox::critical(
@@ -562,9 +678,7 @@ void ConfigDialog::ClearUnusedUserPrediction() {
     return;
   }
 
-#ifndef OS_MACOSX
   client_->CheckVersionOrRestartServer();
-#endif  // OS_MACOSX
 
   if (!client_->ClearUnusedUserPrediction()) {
     QMessageBox::critical(
@@ -607,6 +721,13 @@ void ConfigDialog::SelectInputModeSetting(int index) {
   editRomanTableButton->setEnabled((index == 0));
 }
 
+void ConfigDialog::SelectAutoConversionSetting(int state) {
+  kutenCheckBox->setEnabled(static_cast<bool>(state));
+  toutenCheckBox->setEnabled(static_cast<bool>(state));
+  questionMarkCheckBox->setEnabled(static_cast<bool>(state));
+  exclamationMarkCheckBox->setEnabled(static_cast<bool>(state));
+}
+
 void ConfigDialog::ResetToDefaults() {
   if (QMessageBox::Ok ==
       QMessageBox::question(
