@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,15 +31,16 @@
 
 #include "session/session_converter.h"
 
+#include <vector>
+
 #include "base/util.h"
 #include "converter/converter_interface.h"
 #include "converter/segments.h"
 #include "composer/composer.h"
-#include "session/config_handler.h"
-#include "session/config.pb.h"
 #include "session/internal/candidate_list.h"
 #include "session/internal/session_normalizer.h"
 #include "session/internal/session_output.h"
+#include "transliteration/transliteration.h"
 
 namespace mozc {
 namespace session {
@@ -63,48 +64,14 @@ SessionConverter::SessionConverter(const ConverterInterface *converter)
   operation_preferences_.candidate_shortcuts.clear();
 }
 
-SessionConverter::~SessionConverter() {
-  // TODO(taku): not sure that Converter::Sync() can be called inside
-  // SessionHandler::DeleteSession. Need to investigate the latency
+SessionConverter::~SessionConverter() {}
 
-  // sync all mutable data to disk.
-  VLOG(1) << "Syncing converter data";
-  converter_->Sync();
-}
-
-void SessionConverter::ReloadConfig() {
-  UpdateConfig(config::ConfigHandler::GetConfig());
-}
-
-void SessionConverter::UpdateConfig(const config::Config &config) {
-  // Keyboard shortcut for candidates.
-  const char kShortcut123456789[] = "123456789";
-  const char kShortcutASDFGHJKL[] = "asdfghjkl";
-  switch (config.selection_shortcut()) {
-    case config::Config::SHORTCUT_123456789:
-      operation_preferences_.candidate_shortcuts = kShortcut123456789;
-      break;
-    case config::Config::SHORTCUT_ASDFGHJKL:
-      operation_preferences_.candidate_shortcuts = kShortcutASDFGHJKL;
-      break;
-    case config::Config::NO_SHORTCUT:
-      operation_preferences_.candidate_shortcuts.clear();
-      break;
-    default:
-      LOG(WARNING) << "Unkown shortcuts type: "
-                   << GET_CONFIG(selection_shortcut);
-      break;
-  }
-
-  // Cascading Window.
-#ifdef OS_LINUX
-  // TODO(komatsu): Move this logic to the client code.
-  operation_preferences_.use_cascading_window = false;
-#else
-  if (config.has_use_cascading_window()) {
-    operation_preferences_.use_cascading_window = config.use_cascading_window();
-  }
-#endif
+void SessionConverter::SetOperationPreferences(
+    const OperationPreferences &preferences) {
+  operation_preferences_.use_cascading_window =
+      preferences.use_cascading_window;
+  operation_preferences_.candidate_shortcuts =
+      preferences.candidate_shortcuts;
 }
 
 bool SessionConverter::CheckState(States states) const {
@@ -136,26 +103,53 @@ bool SessionConverter::ConvertWithPreferences(
   segments_->set_request_type(Segments::CONVERSION);
   SetConversionPreferences(preferences, segments_.get());
 
-  string preedit;
-  composer_->GetQueryForConversion(&preedit);
-  if (!converter_->StartConversion(segments_.get(), preedit)) {
-    LOG(WARNING) << "StartConversion() failed";
+  if (!converter_->StartConversionWithComposer(segments_.get(), composer_)) {
+    LOG(WARNING) << "StartConversionWithComposer() failed";
     return false;
   }
 
   segment_index_ = 0;
   state_ = CONVERSION;
-  UpdateCandidateList();
   candidate_list_visible_ = false;
+  UpdateCandidateList();
   GetPreeditAndConversion(0, segments_->conversion_segments_size(),
                           &composition_, &default_result_);
   return true;
 }
 
+bool SessionConverter::ConvertReverse(const string &source_text,
+                                      composer::Composer *composer) {
+  Segments reverse_segments;
+  if (!converter_->StartReverseConversion(&reverse_segments, source_text)) {
+    return false;
+  }
+  if (reverse_segments.segments_size() == 0) {
+    LOG(WARNING) << "no segments from reverse conversion";
+    return false;
+  }
+  string reading;
+  for (int i = 0; i < reverse_segments.segments_size(); ++i) {
+    const mozc::Segment &segment = reverse_segments.segment(i);
+    if (segment.candidates_size() == 0) {
+      LOG(WARNING) << "got an empty segment from reverse conversion";
+      return false;
+    }
+    reading.append(segment.candidate(0).value);
+  }
+
+  composer->InsertCharacterPreedit(reading);
+  // start conversion here.
+  if (!Convert(composer)) {
+    LOG(ERROR) << "Failed to start conversion for reverse conversion";
+    return false;
+  }
+  return true;
+}
+
 namespace {
 Attributes GetT13nAttributes(const transliteration::TransliterationType type) {
   Attributes attributes = NO_ATTRIBUTES;
-  switch(type) {
+  switch (type) {
     case transliteration::HIRAGANA:  // "ひらがな"
       attributes = HIRAGANA;
       break;
@@ -223,7 +217,8 @@ bool SessionConverter::ConvertToTransliteration(
     // converter/converter.cc to enable to accept mozc::Segment::FIXED
     // from the session layer.
     if (segments_->conversion_segments_size() != 1) {
-      converter_->ResizeSegment(segments_.get(), 0, Util::CharsLen(composition_));
+      converter_->ResizeSegment(segments_.get(), 0,
+                                Util::CharsLen(composition_));
       UpdateCandidateList();
     }
 
@@ -262,13 +257,13 @@ bool SessionConverter::ConvertToHalfWidth(const composer::Composer *composer) {
       LOG(ERROR) << "Conversion failed";
       return false;
     }
-
     // TODO(komatsu): This is a workaround to transliterate the whole
     // preedit as a single segment.  We should modify
     // converter/converter.cc to enable to accept mozc::Segment::FIXED
     // from the session layer.
     if (segments_->conversion_segments_size() != 1) {
-      converter_->ResizeSegment(segments_.get(), 0, Util::CharsLen(composition_));
+      converter_->ResizeSegment(segments_.get(), 0,
+                                Util::CharsLen(composition_));
       UpdateCandidateList();
     }
 
@@ -283,7 +278,9 @@ bool SessionConverter::ConvertToHalfWidth(const composer::Composer *composer) {
   // If composition_ is "あｂｃ", it should be treated as Katakana.
   if (Util::ContainsScriptType(*composition, Util::KATAKANA) ||
       Util::ContainsScriptType(*composition, Util::HIRAGANA) ||
-      Util::ContainsScriptType(*composition, Util::KANJI)) {
+      Util::ContainsScriptType(*composition, Util::KANJI) ||
+      Util::IsKanaSymbolContained(*composition)
+  ) {
     attributes |= KATAKANA;
   } else {
     attributes |= ASCII;
@@ -317,7 +314,8 @@ bool SessionConverter::SwitchKanaType(const composer::Composer *composer) {
     // converter/converter.cc to enable to accept mozc::Segment::FIXED
     // from the session layer.
     if (segments_->conversion_segments_size() != 1) {
-      converter_->ResizeSegment(segments_.get(), 0, Util::CharsLen(composition_));
+      converter_->ResizeSegment(segments_.get(), 0,
+                                Util::CharsLen(composition_));
       UpdateCandidateList();
     }
 
@@ -363,11 +361,11 @@ void PrependCandidates(const vector<Segment::Candidate> &candidates,
   if (segments->conversion_segments_size() == 0) {
     segments->clear_conversion_segments();
     Segment *segment = segments->add_segment();
-    segment->clear();
+    segment->Clear();
     segment->set_key(preedit);
   }
 
-  DCHECK(segments->conversion_segments_size() == 1);
+  DCHECK_EQ(1, segments->conversion_segments_size());
   Segment *segment = segments->mutable_conversion_segment(0);
   DCHECK(segment);
 
@@ -414,7 +412,7 @@ bool SessionConverter::SuggestWithPreferences(
     converter_->CancelConversion(segments_.get());
     return false;
   }
-  DCHECK(segments_->conversion_segments_size() == 1);
+  DCHECK_EQ(1, segments_->conversion_segments_size());
 
   // Copy current suggestions so that we can merge
   // prediction/suggestions later
@@ -575,7 +573,7 @@ void SessionConverter::CommitSuggestion(const size_t index) {
 
 void SessionConverter::CommitFirstSegment(composer::Composer *composer) {
   DCHECK(CheckState(PREDICTION | CONVERSION));
-  DCHECK(composer == composer_);
+  DCHECK_EQ(composer, composer_);
   ResetResult();
   candidate_list_visible_ = false;
 
@@ -600,7 +598,7 @@ void SessionConverter::CommitFirstSegment(composer::Composer *composer) {
     composer->DeleteAt(0);
   }
   // The number of segments should be more than one.
-  DCHECK(composer->GetLength() > 0);
+  DCHECK_GT(composer->GetLength(), 0);
 
   // Adjust the segment_index, since the first segment disappeared.
   if (segment_index_ > 0) {
@@ -641,13 +639,15 @@ void SessionConverter::SegmentFocusRight() {
     return;  // Do nothing.
   }
   ResetResult();
+  SegmentFix();
 
   if (segment_index_ + 1 >= segments_->conversion_segments_size()) {
-    return;
+    // If |segment_index_| is at the tail of the segments,
+    // focus on the head.
+    segment_index_ = 0;
+  } else {
+    ++segment_index_;
   }
-
-  SegmentFix();
-  ++segment_index_;
   UpdateCandidateList();
 }
 
@@ -676,13 +676,16 @@ void SessionConverter::SegmentFocusLeft() {
     return;  // Do nothing.
   }
   ResetResult();
+  SegmentFix();
 
   if (segment_index_ <= 0) {
-    return;
+    // If |segment_index_| is at the head of the segments,
+    // focus on the tail.
+    segment_index_ = segments_->conversion_segments_size() - 1;
+  } else {
+    --segment_index_;
   }
 
-  SegmentFix();
-  --segment_index_;
   UpdateCandidateList();
 }
 
@@ -703,6 +706,22 @@ void SessionConverter::SegmentFocusLeftEdge() {
   UpdateCandidateList();
 }
 
+void SessionConverter::SegmentFocusRightOrCommit() {
+  DCHECK(CheckState(PREDICTION | CONVERSION));
+  candidate_list_visible_ = false;
+
+  if (segment_index_ + 1 >= segments_->conversion_segments_size()) {
+    // If |segment_index_| is at the tail of the segments,
+    // commit all segments.
+    Commit();
+  } else {
+    ResetResult();
+    SegmentFix();
+    ++segment_index_;
+    UpdateCandidateList();
+  }
+}
+
 void SessionConverter::SegmentWidthExpand() {
   DCHECK(CheckState(PREDICTION | CONVERSION));
   candidate_list_visible_ = false;
@@ -852,8 +871,13 @@ void SessionConverter::FillOutput(commands::Output *output) const {
 
   // Composition on Suggestion
   if (CheckState(SUGGESTION)) {
-    DCHECK(composer_ != NULL);
-    session::SessionOutput::FillPreedit(*composer_, output->mutable_preedit());
+    DCHECK(composer_);
+    // When the suggestion comes from zero query suggestion, the
+    // composer is empty.  In that case, preedit is not rendered.
+    if (!composer_->Empty()) {
+      session::SessionOutput::FillPreedit(*composer_,
+                                          output->mutable_preedit());
+    }
   } else if (CheckState(PREDICTION | CONVERSION)) {
     // Conversion on Prediction or Conversion
     FillConversion(output->mutable_preedit());
@@ -863,6 +887,54 @@ void SessionConverter::FillOutput(commands::Output *output) const {
       candidate_list_visible_) {
     FillCandidates(output->mutable_candidates());
   }
+  // All candidate words
+  if (CheckState(SUGGESTION | PREDICTION | CONVERSION)) {
+    FillAllCandidateWords(output->mutable_all_candidate_words());
+  }
+}
+
+void SessionConverter::FillContext(commands::Context *context) const {
+  if (context->has_preceding_text()) {
+    // Client has set the information of surrounding text, so do nothing.
+    return;
+  }
+  if (segments_->history_segments_size() == 0) {
+    return;
+  }
+
+  // Set preceding text using history segments.
+  string *preceding_text = context->mutable_preceding_text();
+  for (size_t i = 0; i < segments_->history_segments_size(); ++i) {
+    *preceding_text += segments_->history_segment(i).candidate(0).value;
+  }
+}
+
+void SessionConverter::GetHistorySegments(vector<string> *history) const {
+  DCHECK(history);
+  history->clear();
+  for (size_t i = 0; i < segments_->history_segments_size(); ++i) {
+    history->push_back(segments_->history_segment(i).candidate(0).value);
+  }
+}
+
+void SessionConverter::SetHistorySegments(const vector<string> &history) {
+  segments_->erase_segments(0, segments_->history_segments_size());
+
+  for (size_t i = 0; i < history.size(); ++i) {
+    Segment *segment = segments_->insert_segment(i);
+    segment->set_segment_type(Segment::HISTORY);
+    segment->set_key(history[i]);
+
+    Segment::Candidate *candidate = segment->add_candidate();
+    candidate->value = history[i];
+    candidate->content_value = history[i];
+    candidate->key = history[i];
+    candidate->content_key = history[i];
+  }
+}
+
+void SessionConverter::RemoveTailOfHistorySegments(size_t num_of_characters) {
+  segments_->RemoveTailOfHistorySegments(num_of_characters);
 }
 
 const string &SessionConverter::GetDefaultResult() const {
@@ -873,11 +945,7 @@ const string &SessionConverter::GetDefaultResult() const {
 void SessionConverter::SetConversionPreferences(
     const ConversionPreferences &preferences,
     Segments *segments) {
-  if (preferences.use_history) {
-    segments->enable_user_history();
-  } else {
-    segments->disable_user_history();
-  }
+  segments->set_user_history_enabled(preferences.use_history);
   segments->set_max_history_segments_size(preferences.max_history_size);
 }
 
@@ -952,20 +1020,26 @@ int GetT13nId(const transliteration::TransliterationType type) {
 
 void SessionConverter::UpdateCandidateList() {
   DCHECK(CheckState(SUGGESTION | PREDICTION | CONVERSION));
-  InitSegment(segment_index_);
   candidate_list_->Clear();
 
   const Segment &segment = segments_->conversion_segment(segment_index_);
   for (size_t i = 0; i < segment.candidates_size(); ++i) {
     candidate_list_->AddCandidate(i, segment.candidate(i).value);
+    // if candidate has spelling correction attribute,
+    // always display the candidate to let user know the
+    // miss spelled candidate.
+    if (i < 10 &&
+        (segment.candidate(i).attributes &
+         Segment::Candidate::SPELLING_CORRECTION)) {
+      candidate_list_visible_ = true;
+    }
   }
 
   const bool focused = (segments_->request_type() != Segments::SUGGESTION);
   candidate_list_->set_focused(focused);
 
-  // If the session status is not conversion (it means suggestion or
-  // prediction), translitaration should not be included.
-  if (segments_->request_type() != Segments::CONVERSION) {
+  if (segment.meta_candidates_size() == 0) {
+    LOG(WARNING) << "T13N is not initialized: " << segment.key();
     return;
   }
 
@@ -996,56 +1070,6 @@ void SessionConverter::UpdateCandidateList() {
   }
 }
 
-// TODO(komatsu): The number should be flexible.
-// This number should be equal to kMaxCandidatesSize of
-// converter/nbest_generator.cc
-static const size_t kMaxCandidatesSize = 200;
-
-void SessionConverter::InitSegment(const size_t segment_index) {
-  DCHECK(CheckState(SUGGESTION | PREDICTION | CONVERSION));
-  Segment *segment = segments_->mutable_conversion_segment(segment_index);
-  if (segment == NULL) {
-    LOG(ERROR) << "The segment is not available: " << segment_index;
-    return;
-  }
-
-  // If the available candidates are already expanded, do nothing any more.
-  if (segment->requested_candidates_size() >= kMaxCandidatesSize) {
-    return;
-  }
-
-  // Initialize candidates at most kMaxCandidatesSize.
-  converter_->GetCandidates(segments_.get(),
-                            segment_index,
-                            kMaxCandidatesSize);
-
-  // Initialize transliterations.
-  if (segment->initialized_transliterations()) {
-    return;
-  }
-
-  size_t composition_pos = 0;
-  for (size_t i = 0; i < segment_index; ++i) {
-    composition_pos += Util::CharsLen(segments_->conversion_segment(i).key());
-  }
-  const size_t composition_size = Util::CharsLen(segment->key());
-
-  vector<string> t13ns;
-  composer_->GetSubTransliterations(composition_pos, composition_size, &t13ns);
-
-  // Normalize the output.  Hiragana VU and wave-dash characters are
-  // normalized to platform preferable characters.
-  vector<string> normalized_t13ns;
-  string normalized;
-  for (size_t i = 0; i < t13ns.size(); ++i) {
-    normalized.clear();
-    SessionNormalizer::NormalizeTransliterationText(t13ns[i], &normalized);
-    normalized_t13ns.push_back(normalized);
-  }
-  segment->SetTransliterations(normalized_t13ns);
-}
-
-
 int SessionConverter::GetCandidateIndexForConverter(
     const size_t segment_index) const {
   DCHECK(CheckState(SUGGESTION | PREDICTION | CONVERSION));
@@ -1141,5 +1165,31 @@ void SessionConverter::FillCandidates(commands::Candidates *candidates) const {
   SessionOutput::FillFooter(candidates->category(), candidates);
 }
 
+
+void SessionConverter::FillAllCandidateWords(
+    commands::CandidateList *candidates) const {
+  DCHECK(CheckState(SUGGESTION | PREDICTION | CONVERSION));
+  commands::Category category;
+  switch (segments_->request_type()) {
+    case Segments::CONVERSION:
+      category = commands::CONVERSION;
+      break;
+    case Segments::PREDICTION:
+      category = commands::PREDICTION;
+      break;
+    case Segments::SUGGESTION:
+      category = commands::SUGGESTION;
+      break;
+    default:
+      LOG(WARNING) << "Unkown request type: " << segments_->request_type();
+      category = commands::CONVERSION;
+      break;
+  }
+
+  const Segment &segment = segments_->conversion_segment(segment_index_);
+  SessionOutput::FillAllCandidateWords(
+      segment, *candidate_list_, category, candidates);
+}
+
 }  // namespace session
 }  // namespace mozc
