@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -35,14 +35,17 @@
 #include <vector>
 
 #include "base/base.h"
+#include "base/util.h"
+#include "base/process.h"
+#include "base/singleton.h"
 #include "base/stopwatch.h"
 #include "converter/converter_interface.h"
-#include "composer/table.h"
 #include "session/commands.pb.h"
 #include "session/config_handler.h"
 #include "session/config.pb.h"
 #include "session/internal/keymap.h"
-#include "session/session.h"
+#include "session/session_factory.h"
+#include "session/session_interface.h"
 #include "session/session_observer_handler.h"
 #include "session/session_watch_dog.h"
 
@@ -76,16 +79,39 @@ DEFINE_bool(restricted, false,
 
 namespace mozc {
 
+namespace {
+bool IsApplicationAlive(const session::SessionInterface *session) {
+  const commands::ApplicationInfo &info = session->application_info();
+  // When the thread/process's current status is unknown, i.e.,
+  // if IsThreadAlive/IsProcessAlive functions failed to know the
+  // status of the thread/process, return true just in case.
+  // Here, we want to kill the session only when the target thread/process
+  // are terminated with 100% probability. Otherwise, it's better to do
+  // nothing to prevent any side effects.
+#ifdef OS_WINDOWS
+  if (info.has_thread_id()) {
+    return Process::IsThreadAlive(
+        static_cast<size_t>(info.thread_id()), true);
+  }
+#else   // OS_WINDOWS
+  if (info.has_process_id()) {
+    return Process::IsProcessAlive(
+        static_cast<size_t>(info.process_id()), true);
+  }
+#endif  // OS_WINDOWS
+  return true;
+}
+}  // namespace
+
 SessionHandler::SessionHandler()
-    : preedit_table_(new composer::Table()),
-      keymap_(new keymap::KeyMapManager()),
-      converter_(NULL),
-      is_available_(false),
+    : is_available_(false),
       keyevent_counter_(0),
       max_session_size_(0),
       last_session_empty_time_(Util::GetTime()),
       last_cleanup_time_(0),
       last_create_session_time_(0),
+      session_factory_(
+          session::SessionFactory::GetDefaultSessionFactory()),
       observer_handler_(new session::SessionObserverHandler()),
       stopwatch_(new Stopwatch) {
   if (FLAGS_restricted) {
@@ -108,12 +134,7 @@ SessionHandler::SessionHandler()
   max_session_size_ = max(2, min(FLAGS_max_session_size, 128));
   session_map_.reset(new SessionMap(max_session_size_));
 
-  if (!preedit_table_->Initialize()) {
-    return;
-  }
-
-  converter_ = ConverterFactory::GetConverter();
-  if (converter_ == NULL) {
+  if (session_factory_ == NULL || !session_factory_->IsAvailable()) {
     return;
   }
 
@@ -147,6 +168,10 @@ bool SessionHandler::StartWatchDog() {
 
 void SessionHandler::ReloadSession() {
   observer_handler_->Reload();
+  ReloadConfig();
+}
+
+void SessionHandler::ReloadConfig() {
   for (SessionElement *element =
            const_cast<SessionElement *>(session_map_->Head());
        element != NULL; element = element->next) {
@@ -158,7 +183,7 @@ void SessionHandler::ReloadSession() {
 
 bool SessionHandler::SyncData(commands::Command *command) {
   VLOG(1) << "Syncing user data";
-  converter_->Sync();
+  ConverterFactory::GetConverter()->Sync();
   command->mutable_output()->set_id(command->input().id());
   return true;
 }
@@ -174,8 +199,7 @@ bool SessionHandler::Shutdown(commands::Command *command) {
 bool SessionHandler::Reload(commands::Command *command) {
   VLOG(1) << "Reloading server";
   ReloadSession();
-  keymap_->Reload();
-  preedit_table_->Reload();
+  session_factory_->Reload();
   RunReloaders();  // call all reloaders defined in .cc file
   command->mutable_output()->set_id(command->input().id());
   return true;
@@ -183,27 +207,31 @@ bool SessionHandler::Reload(commands::Command *command) {
 
 bool SessionHandler::ClearUserHistory(commands::Command *command) {
   VLOG(1) << "Clearing user history";
-  converter_->ClearUserHistory();
+  ConverterFactory::GetConverter()->ClearUserHistory();
   command->mutable_output()->set_id(command->input().id());
   return true;
 }
 
 bool SessionHandler::ClearUserPrediction(commands::Command *command) {
   VLOG(1) << "Clearing user prediction";
-  converter_->ClearUserPrediction();
+  ConverterFactory::GetConverter()->ClearUserPrediction();
   command->mutable_output()->set_id(command->input().id());
   return true;
 }
 
 bool SessionHandler::ClearUnusedUserPrediction(commands::Command *command) {
   VLOG(1) << "Clearing unused user prediction";
-  converter_->ClearUnusedUserPrediction();
+  ConverterFactory::GetConverter()->ClearUnusedUserPrediction();
   command->mutable_output()->set_id(command->input().id());
   return true;
 }
 
 bool SessionHandler::GetConfig(commands::Command *command) {
   VLOG(1) << "Getting user config";
+  // Ensure the onmemory config is same as the locally stored one
+  // because the local data could be changed by sync.
+  ReloadConfig();
+
   if (!config::ConfigHandler::GetConfig(
           command->mutable_output()->mutable_config())) {
     LOG(WARNING) << "cannot get config";
@@ -233,6 +261,7 @@ bool SessionHandler::SetConfig(commands::Command *command) {
   return true;
 }
 
+
 bool SessionHandler::EvalCommand(commands::Command *command) {
   if (!is_available_) {
     return false;
@@ -315,8 +344,13 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
   return is_available_;
 }
 
-Session *SessionHandler::NewSession() {
-  return new Session(preedit_table_.get(), converter_, keymap_.get());
+session::SessionInterface *SessionHandler::NewSession() {
+  return session_factory_->NewSession();
+}
+
+void SessionHandler::SetSessionFactory(
+    session::SessionFactoryInterface *new_factory) {
+  session_factory_ = new_factory;
 }
 
 void SessionHandler::AddObserver(session::SessionObserverInterface *observer) {
@@ -327,7 +361,7 @@ bool SessionHandler::SendKey(commands::Command *command) {
   const SessionID id = command->input().id();
   command->mutable_output()->set_id(id);
 
-  Session **session = session_map_->MutableLookup(id);
+  session::SessionInterface **session = session_map_->MutableLookup(id);
   if (session == NULL || *session == NULL) {
     LOG(WARNING) << "SessionID " << id << " is not available";
     return false;
@@ -339,7 +373,7 @@ bool SessionHandler::SendKey(commands::Command *command) {
 bool SessionHandler::TestSendKey(commands::Command *command) {
   const SessionID id = command->input().id();
   command->mutable_output()->set_id(id);
-  Session **session = session_map_->MutableLookup(id);
+  session::SessionInterface **session = session_map_->MutableLookup(id);
   if (session == NULL || *session == NULL) {
     LOG(WARNING) << "SessionID " << id << " is not available";
     return false;
@@ -351,7 +385,8 @@ bool SessionHandler::TestSendKey(commands::Command *command) {
 bool SessionHandler::SendCommand(commands::Command *command) {
   const SessionID id = command->input().id();
   command->mutable_output()->set_id(id);
-  Session **session = const_cast<Session **>(session_map_->Lookup(id));
+  session::SessionInterface **session =
+    const_cast<session::SessionInterface **>(session_map_->Lookup(id));
   if (session == NULL || *session == NULL) {
     LOG(WARNING) << "SessionID " << id << " is not available";
     return false;
@@ -391,7 +426,7 @@ bool SessionHandler::CreateSession(commands::Command *command) {
             << oldest_element->key << " is removed";
   };
 
-  Session *session = NewSession();
+  session::SessionInterface *session = NewSession();
   if (session == NULL) {
     LOG(ERROR) << "Cannot allocate new Session";
     return false;
@@ -405,6 +440,18 @@ bool SessionHandler::CreateSession(commands::Command *command) {
   // The oldes item should be reused
   DCHECK(oldest_element == NULL || oldest_element == element);
 
+  if (command->input().has_capability()) {
+    session->set_client_capability(command->input().capability());
+  }
+
+  if (command->input().has_application_info()) {
+    session->set_application_info(command->input().application_info());
+  }
+
+  // Ensure the onmemory config is same as the locally stored one
+  // because the local data could be changed by sync.
+  ReloadConfig();
+
   // session is not empty.
   last_session_empty_time_ = 0;
 
@@ -415,12 +462,14 @@ bool SessionHandler::DeleteSession(commands::Command *command) {
   const SessionID id = command->input().id();
   command->mutable_output()->set_id(id);
   DeleteSessionID(id);
+  ConverterFactory::GetConverter()->Sync();
   return true;
 }
 
 // Scan all sessions and find and delete session which is either
 // (a) The session is not activated for 60min
 // (b) The session is created but not accessed for 5min
+// (c) application is already terminated.
 // Also, if timeout is enabled, shutdown server if there is
 // no active session and client doesn't send any conversion
 // request to the server for FLAGS_timeout sec.
@@ -456,8 +505,11 @@ bool SessionHandler::Cleanup(commands::Command *command) {
   for (SessionElement *element =
            const_cast<SessionElement *>(session_map_->Head());
        element != NULL; element = element->next) {
-    Session *session = element->value;
-    if (session->last_command_time() == 0) {
+    session::SessionInterface *session = element->value;
+    if (!IsApplicationAlive(session)) {
+      VLOG(2) << "Application is not alive. Removing: " << element->key;
+      remove_ids.push_back(element->key);
+    } else if (session->last_command_time() == 0) {
       // no command is exectuted
       if ((current_time - session->create_session_time()) >=
           kCreateSessionTimeout) {
@@ -476,6 +528,10 @@ bool SessionHandler::Cleanup(commands::Command *command) {
     VLOG(1) << "Session ID " << remove_ids[i] << " is removed by server";
   }
 
+  // Sync all data.
+  // This is a regression bug fix http://b/issue?id=3033708
+  ConverterFactory::GetConverter()->Sync();
+
   // timeout is enabled.
   if (FLAGS_timeout > 0 &&
       last_session_empty_time_ != 0 &&
@@ -518,7 +574,7 @@ SessionID SessionHandler::CreateNewSessionID() {
 }
 
 bool SessionHandler::DeleteSessionID(SessionID id) {
-  Session **session = session_map_->MutableLookup(id);
+  session::SessionInterface **session = session_map_->MutableLookup(id);
   if (session == NULL || *session == NULL) {
     LOG_IF(WARNING, id != 0) << "cannot find SessionID " << id;
     return false;
