@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -40,8 +40,10 @@
 #include "base/protobuf/message.h"
 #include "base/singleton.h"
 #include "base/util.h"
+#include "session/commands.pb.h"
 #include "session/config.pb.h"
 #include "session/ime_switch_util.h"
+#include "unix/ibus/config_util.h"
 #include "unix/ibus/engine_registrar.h"
 #include "unix/ibus/key_translator.h"
 #include "unix/ibus/mozc_engine_property.h"
@@ -65,12 +67,17 @@ const int32 kBadCandidateId = -1;
 const char kMozcSectionName[] = "engine/Mozc";
 
 // Icon path for MozcTool
-// TODO(taku): currently, unknown icon is displayed.
-const char kMozcToolIconPath[] = "unknown.ico";
+const char kMozcToolIconPath[] = "tool.png";
 
 // for every 5 minutes, call SyncData
 const uint64 kSyncDataInterval = 5 * 60;
 
+// Backspace key code
+const guint kBackSpaceKeyCode = 14;
+
+// Left shift key code
+const guint kShiftLeftKeyCode = 42;
+
 uint64 GetTime() {
   return static_cast<uint64>(time(NULL));
 }
@@ -207,18 +214,41 @@ int CursorPos(const commands::Output &output) {
 }
 
 // Returns an IBusText used for showing the auxiliary text in the candidate
-// window.
-// Caller must release the returned IBusText object.
+// window. Returns NULL if no text has to be shown. Caller must release the
+// returned IBusText object.
 IBusText *ComposeAuxiliaryText(const commands::Candidates &candidates) {
-  // Max size of candidates is 200 so 128 is sufficient size for the buffer.
-  char buf[128];
-  const int result = snprintf(buf,
-                              128,
-                              "%d/%d",
-                              candidates.focused_index() + 1,
-                              candidates.size());
-  DCHECK_GE(result, 0) << "snprintf in ComposeAuxiliaryText failed";
-  return ibus_text_new_from_string(buf);
+  if (!candidates.has_footer()) {
+    // We don't have to show the auxiliary text.
+    return NULL;
+  }
+  const commands::Footer &footer = candidates.footer();
+
+  std::string auxiliary_text;
+  if (footer.has_label()) {
+    // TODO(yusukes,mozc-team): label() is not localized. Currently, it's always
+    // written in Japanese (in UTF-8).
+    auxiliary_text = footer.label();
+  } else if (footer.has_sub_label()) {
+    // Windows client shows sub_label() only when label() is not specified. We
+    // follow the policy.
+    auxiliary_text = footer.sub_label();
+  }
+
+  if (footer.has_index_visible() && footer.index_visible() &&
+      candidates.has_focused_index()) {
+    // Max size of candidates is 200 so 128 is sufficient size for the buffer.
+    char index_buf[128] = {0};
+    const int result = snprintf(index_buf,
+                                sizeof(index_buf) - 1,
+                                "%s%d/%d",
+                                (auxiliary_text.empty() ? "" : " "),
+                                candidates.focused_index() + 1,
+                                candidates.size());
+    DCHECK_GE(result, 0) << "snprintf in ComposeAuxiliaryText failed";
+    auxiliary_text += index_buf;
+  }
+  return auxiliary_text.empty() ?
+      NULL : ibus_text_new_from_string(auxiliary_text.c_str());
 }
 
 MozcEngine::MozcEngine()
@@ -233,20 +263,21 @@ MozcEngine::MozcEngine()
       prop_composition_mode_(NULL),
       prop_mozc_tool_(NULL),
       current_composition_mode_(kMozcEngineInitialCompositionMode),
-      preedit_method_(config::Config::ROMAN) {
+      preedit_method_(config::Config::ROMAN),
+      ignore_reset_for_deletion_range_workaround_(false) {
   // |sub_prop_list| is a radio menu which is shown when a button in the
   // language panel (i.e. |prop_composition_mode_| below) is clicked.
   IBusPropList *sub_prop_list = ibus_prop_list_new();
 
   // Create items for the radio menu.
-  IBusText *label_for_panel = NULL;  // e.g. Hiragana letter A.
+  string icon_path_for_panel;
   for (size_t i = 0; i < kMozcEnginePropertiesSize; ++i) {
     const MozcEngineProperty &entry = kMozcEngineProperties[i];
     IBusText *label = ibus_text_new_from_static_string(entry.label);
     IBusPropState state = PROP_STATE_UNCHECKED;
     if (entry.composition_mode == kMozcEngineInitialCompositionMode) {
       state = PROP_STATE_CHECKED;
-      label_for_panel = ibus_text_new_from_static_string(entry.label_for_panel);
+      icon_path_for_panel = GetIconPath(entry.icon);
     }
     IBusProperty *item = ibus_property_new(entry.key,
                                            PROP_TYPE_RADIO,
@@ -261,13 +292,13 @@ MozcEngine::MozcEngine()
     ibus_prop_list_append(sub_prop_list, item);
     // |sub_prop_list| owns |item| by calling g_object_ref_sink for the |item|.
   }
-  DCHECK(label_for_panel) << "All items are disabled by default";
+  DCHECK(!icon_path_for_panel.empty());
 
   // The label of |prop_composition_mode_| is shown in the language panel.
   prop_composition_mode_ = ibus_property_new("CompositionMode",
                                              PROP_TYPE_MENU,
-                                             label_for_panel,
-                                             NULL /* icon */,
+                                             NULL /* label */,
+                                             icon_path_for_panel.c_str(),
                                              NULL /* tooltip */,
                                              TRUE /* sensitive */,
                                              TRUE /* visible */,
@@ -287,11 +318,12 @@ MozcEngine::MozcEngine()
     for (size_t i = 0; i < kMozcEngineToolPropertiesSize; ++i) {
       const MozcEngineToolProperty &entry = kMozcEngineToolProperties[i];
       IBusText *label = ibus_text_new_from_static_string(entry.label);
+      // TODO(yusukes): It would be better to use entry.icon here?
       IBusProperty *item = ibus_property_new(entry.mode,
                                              PROP_TYPE_NORMAL,
                                              label,
-                                             NULL,
-                                             NULL,
+                                             NULL /* icon */,
+                                             NULL /* tooltip */,
                                              TRUE,
                                              TRUE,
                                              PROP_STATE_UNCHECKED,
@@ -331,6 +363,17 @@ MozcEngine::MozcEngine()
     ibus_prop_list_append(prop_root_, prop_mozc_tool_);
   }
 #endif
+
+  // We don't enable the DELETE_PRECEDING_TEXT feature on Chrome OS for now
+  // since it's highly experimental.
+#ifndef OS_CHROMEOS
+  // Currently client capability is fixed.
+  commands::Capability capability;
+  capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
+  session_->set_client_capability(capability);
+  // TODO(yusukes): write a unit test to check if the capability is set
+  // as expected.
+#endif
 }
 
 MozcEngine::~MozcEngine() {
@@ -381,34 +424,48 @@ void MozcEngine::CursorUp(IBusEngine *engine) {
 }
 
 void MozcEngine::Disable(IBusEngine *engine) {
-  // TODO(mazda): Implement this.
+  // Stop ignoring "reset" signal.  See ProcessKeyevent().
+  ignore_reset_for_deletion_range_workaround_ = false;
+
+  RevertSession(engine);
 }
 
 void MozcEngine::Enable(IBusEngine *engine) {
+  // Stop ignoring "reset" signal.  See ProcessKeyevent().
+  ignore_reset_for_deletion_range_workaround_ = false;
+
   // Launch mozc_server
   session_->EnsureConnection();
   UpdatePreeditMethod();
+
+  // When ibus-mozc is disabled by the "next input method" hot key, ibus-daemon
+  // does not call MozcEngine::Disable(). Call RevertSession() here so the
+  // mozc_server could discard a preedit string before the hot key is pressed
+  // (crosbug.com/4596).
+  RevertSession(engine);
 }
 
 void MozcEngine::FocusIn(IBusEngine *engine) {
   ibus_engine_register_properties(engine, prop_root_);
+  UpdatePreeditMethod();
 }
 
 void MozcEngine::FocusOut(IBusEngine *engine) {
-  commands::SessionCommand command;
-  command.set_type(commands::SessionCommand::REVERT);
-  commands::Output output;
-  session_->SendCommand(command, &output);
-  UpdateAll(engine, output);
+  // Stop ignoring "reset" signal.  See ProcessKeyevent().
+  ignore_reset_for_deletion_range_workaround_ = false;
+
+  RevertSession(engine);
   SyncData(false);
 }
 
 void MozcEngine::PageDown(IBusEngine *engine) {
-  // TODO(mazda): Implement this.
+  // TODO(mazda,yusukes): Implement this to support arrow icons inside the Gtk+
+  // candidate window.
 }
 
 void MozcEngine::PageUp(IBusEngine *engine) {
-  // TODO(mazda): Implement this.
+  // TODO(mazda,yusukes): Implement this to support arrow icons inside the Gtk+
+  // candidate window.
 }
 
 gboolean MozcEngine::ProcessKeyEvent(
@@ -420,9 +477,8 @@ gboolean MozcEngine::ProcessKeyEvent(
           << ", keycode: " << keycode
           << ", modifiers: " << modifiers;
 
-  if (modifiers & IBUS_RELEASE_MASK) {
-    return FALSE;
-  }
+  // Stop ignoring "reset" signal.  See the code of deletion below.
+  ignore_reset_for_deletion_range_workaround_ = false;
 
   // Since IBus for ChromeOS is based on in-process conversion,
   // it is basically ok to call GetConfig() at every keyevent.
@@ -439,7 +495,7 @@ gboolean MozcEngine::ProcessKeyEvent(
 
   // TODO(yusukes): use |layout| in IBusEngineDesc if possible.
   const bool layout_is_jp =
-      !g_strcmp0(ibus_engine_get_name(engine), kEngineNames[1]);
+      !g_strcmp0(ibus_engine_get_name(engine), "mozc-jp");
 
   commands::KeyEvent key;
   if (!key_translator_->Translate(
@@ -448,8 +504,15 @@ gboolean MozcEngine::ProcessKeyEvent(
     return FALSE;
   }
 
-  VLOG(2) << key.DebugString();
+  if (!ProcessModifiers((modifiers & IBUS_RELEASE_MASK) != 0,
+                        keyval,
+                        &key,
+                        &currently_pressed_modifiers_,
+                        &modifiers_to_be_sent_)) {
+    return FALSE;
+  }
 
+  VLOG(2) << key.DebugString();
   if ((current_composition_mode_ == commands::DIRECT) &&
       // We DO consume keys that enable Mozc such as Henkan even when in the
       // DIRECT mode.
@@ -465,13 +528,54 @@ gboolean MozcEngine::ProcessKeyEvent(
 
   VLOG(2) << output.DebugString();
 
+  if (output.has_deletion_range() &&
+      output.deletion_range().offset() < 0 &&
+      output.deletion_range().offset() + output.deletion_range().length() >=
+          0) {
+    // Delete some characters of preceding text.  We really want to use
+    // ibus_engine_delete_surrounding_text(), but it does not work on many
+    // applications, e.g. Chrome, Firefox.  So we currently forward backspaces
+    // to application.
+    // NOTE: There are some workarounds.  They must be maintained
+    // continuosly.
+    // NOTE: It cannot delete range of characters not adjacent to the left side
+    // of cursor.  If the range is not adjacent to the cursor, ignore it.  If
+    // the range contains the cursor, only characters on the left side of cursor
+    // are deleted.
+    const int length = -output.deletion_range().offset();
+
+    // Some applications, e.g. Chrome, delete preedit character when we forward
+    // a backspace.  We can avoid it by hiding preedit before forwarding
+    // backspaces.
+    ibus_engine_hide_preedit_text(engine);
+
+    // Forward backspaces.
+    for (size_t i = 0; i < length; ++i) {
+      ibus_engine_forward_key_event(
+          engine, IBUS_BackSpace, kBackSpaceKeyCode, 0);
+    }
+
+    // This is a workaround.  Some applications (e.g. Chrome, etc.) send
+    // strange signals such as "focus_out" and "focus_in" after forwarding
+    // Backspace key event.  However, such strange signals are not sent if we
+    // forward another key event after backspaces.  Here we choose Shift L,
+    // which is one of the most quiet key event.
+    ibus_engine_forward_key_event(engine, IBUS_Shift_L, kShiftLeftKeyCode, 0);
+
+    // This is also a workaround.  Some applications (e.g. gedit, etc.) send a
+    // "reset" signal after forwarding backspaces.  This signal is sent after
+    // this ProcessKeyEvent() returns, and we want to avoid reverting the
+    // session.
+    // See also Reset(), which calls RevertSession() only if
+    // ignore_reset_for_deletion_range_workaround_ is false.  This flag is
+    // reset on Disable(), FocusOut(), and ProcessKeyEvent().  Reset() also
+    // reset this flag, i.e. this flag does not work more than once.
+    ignore_reset_for_deletion_range_workaround_ = true;
+  }
+
   UpdateAll(engine, output);
 
   const bool consumed = output.consumed();
-  // TODO(mazda): Check if this code is necessary
-  // if (!consumed) {
-  //   ibus_engine_forward_key_event(engine, keyval, keycode, modifiers);
-  //  }
   return consumed ? TRUE : FALSE;
 }
 
@@ -499,17 +603,18 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
   IBusProperty *prop = NULL;
 
 #ifndef OS_CHROMEOS
-  DCHECK(prop_mozc_tool_);
-  while (prop = ibus_prop_list_get(prop_mozc_tool_->sub_props, i++)) {
-    if (!g_strcmp0(property_name, prop->key)) {
-      const MozcEngineToolProperty *entry =
-          reinterpret_cast<const MozcEngineToolProperty*>(
-              g_object_get_data(G_OBJECT(prop), kGObjectDataKey));
-      DCHECK(entry->mode);
-      if (!session_->LaunchTool(entry->mode, "")) {
-        LOG(ERROR) << "cannot launch: " << entry->mode;
+  if (prop_mozc_tool_) {
+    while (prop = ibus_prop_list_get(prop_mozc_tool_->sub_props, i++)) {
+      if (!g_strcmp0(property_name, prop->key)) {
+        const MozcEngineToolProperty *entry =
+            reinterpret_cast<const MozcEngineToolProperty*>(
+                g_object_get_data(G_OBJECT(prop), kGObjectDataKey));
+        DCHECK(entry->mode);
+        if (!session_->LaunchTool(entry->mode, "")) {
+          LOG(ERROR) << "cannot launch: " << entry->mode;
+        }
+        return;
       }
-      return;
     }
   }
 #endif
@@ -529,9 +634,8 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
         // Update Mozc state.
         SetCompositionMode(engine, entry->composition_mode);
         // Update the language panel.
-        ibus_property_set_label(
-            prop_composition_mode_,
-            ibus_text_new_from_static_string(entry->label_for_panel));
+        ibus_property_set_icon(prop_composition_mode_,
+                               GetIconPath(entry->icon).c_str());
       }
       // Update the radio menu item.
       ibus_property_set_state(prop, PROP_STATE_CHECKED);
@@ -545,21 +649,33 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
 
 void MozcEngine::PropertyHide(IBusEngine *engine,
                               const gchar *property_name) {
-  // TODO(mazda): Implement this.
+  // We can ignore the signal.
 }
 
 void MozcEngine::PropertyShow(IBusEngine *engine,
                               const gchar *property_name) {
-  // TODO(mazda): Implement this.
+  // We can ignore the signal.
 }
 
 void MozcEngine::Reset(IBusEngine *engine) {
-  // TODO(mazda): Implement this.
+  // Ignore this signal if ignore_reset_for_deletion_range_workaround_ is true.
+  // This is workaround of deletion of surrounding text.
+  // See also ProcessKeyEvent().
+  if (ignore_reset_for_deletion_range_workaround_) {
+    VLOG(2) << "Reset signal is ignored";
+    // We currently do not know the cases that application sends reset signal
+    // more than once, so reset the flag here.
+    ignore_reset_for_deletion_range_workaround_ = false;
+    return;
+  }
+  RevertSession(engine);
 }
 
 void MozcEngine::SetCapabilities(IBusEngine *engine,
                                  guint capabilities) {
-  // TODO(mazda): Implement this.
+  // TODO(mazda,yusukes): For now, there's nothing to do here. If mozc starts to
+  // support the reconversion feature, we should checkIBUS_CAP_SURROUNDING_TEXT
+  // here to see if the current client can provide surrounding text information.
 }
 
 void MozcEngine::SetCursorLocation(IBusEngine *engine,
@@ -567,7 +683,7 @@ void MozcEngine::SetCursorLocation(IBusEngine *engine,
                                    gint y,
                                    gint w,
                                    gint h) {
-  // TODO(mazda): Implement this.
+  // We can ignore the signal.
 }
 
 GType MozcEngine::GetType() {
@@ -601,29 +717,110 @@ void MozcEngine::Disconnected(IBusBus *bus, gpointer user_data) {
   ibus_quit();
 }
 
-// static
+#ifdef OS_CHROMEOS
+#if IBUS_CHECK_VERSION(1, 3, 99)
+// For IBus 1.4.
 void MozcEngine::ConfigValueChanged(IBusConfig *config,
                                     const gchar *section,
                                     const gchar *name,
-                                    GValue *gvalue,
+                                    GVariant *value,
                                     gpointer user_data) {
-#ifdef OS_CHROMEOS
   // This function might be called _before_ MozcEngineClassInit is called if
   // you press the "Configure..." button for Mozc before switching to the Mozc
   // input method.
   MozcEngine *engine = mozc::Singleton<MozcEngine>::get();
-  engine->UpdateConfig(section, name, gvalue);
+  engine->UpdateConfig(section, name, value);
+}
 #else
-  // On plain Linux, we don't use ibus-gconf for now. In other words, this
-  // method should never be called.
+// For IBus 1.2 and 1.3.
+void MozcEngine::ConfigValueChanged(IBusConfig *config,
+                                    const gchar *section,
+                                    const gchar *name,
+                                    GValue *value,
+                                    gpointer user_data) {
+  MozcEngine *engine = mozc::Singleton<MozcEngine>::get();
+  engine->UpdateConfig(section, name, value);
+}
 #endif
+#endif  // OS_CHROMEOS
+
+// static
+bool MozcEngine::ProcessModifiers(
+    bool is_key_up,
+    gint keyval,
+    commands::KeyEvent *key,
+    set<gint> *currently_pressed_modifiers,
+    set<commands::KeyEvent::ModifierKey> *modifiers_to_be_sent) {
+  // Manage modifier key event.
+  // Modifier key event is sent on key up if non-modifier key has not been
+  // pressed since key down of modifier keys and no modifier keys are pressed
+  // anymore.
+  // Following examples are expected behaviors.
+  //
+  // E.g.) Usual key is sent on key down.  Modifier keys are not sent if usual
+  //       key is sent.
+  //    <Event from ibus> <Event to server>
+  //     Shift down      | None
+  //     "a" down        | Shift+a
+  //     "a" up          | None
+  //     Shift up        | None
+  //
+  // E.g.) Modifier key is sent on key up.
+  //    <Event from ibus> <Event to server>
+  //     Shift down      | None
+  //     Shift up        | Shift
+  //
+  // E.g.) Multiple modifier keys are sent on the last key up.
+  //    <Event from ibus> <Event to server>
+  //     Shift down      | None
+  //     Control down    | None
+  //     Shift up        | None
+  //     Control up      | Control+Shift
+  const bool is_modifier_only =
+      !(key->has_key_code() || key->has_special_key());
+  if (is_key_up) {
+    if (!is_modifier_only) {
+      return false;
+    }
+    currently_pressed_modifiers->erase(keyval);
+    if (!currently_pressed_modifiers->empty() ||
+        modifiers_to_be_sent->empty()) {
+      return false;
+    }
+    // Modifier key event fires
+    key->mutable_modifier_keys()->Clear();
+    for (set<commands::KeyEvent::ModifierKey>::const_iterator it =
+             modifiers_to_be_sent->begin();
+         it != modifiers_to_be_sent->end();
+         ++it) {
+      key->add_modifier_keys(*it);
+    }
+    modifiers_to_be_sent->clear();
+  } else if (is_modifier_only) {
+    if (currently_pressed_modifiers->empty() ||
+        !modifiers_to_be_sent->empty()) {
+      for (size_t i = 0; i < key->modifier_keys_size(); ++i) {
+        modifiers_to_be_sent->insert(key->modifier_keys(i));
+      }
+    }
+    currently_pressed_modifiers->insert(keyval);
+    return false;
+  }
+  // Clear modifier data just in case if |key| has no modifier keys.
+  if (key->modifier_keys_size() == 0) {
+    currently_pressed_modifiers->clear();
+    modifiers_to_be_sent->clear();
+  }
+  return true;
 }
 
 bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
   UpdateResult(engine, output);
   UpdatePreedit(engine, output);
   UpdateCandidates(engine, output);
-  UpdateCompositionMode(engine, output);
+  if (output.has_mode()) {
+    UpdateCompositionMode(engine, output.mode());
+  }
   return true;
 }
 
@@ -661,7 +858,6 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
     return true;
   }
 
-  const guint kPageSize = 9;
   const gboolean kRound = TRUE;
   const commands::Candidates &candidates = output.candidates();
   const gboolean cursor_visible = candidates.has_focused_index() ?
@@ -679,13 +875,52 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
                                                  cursor_visible,
                                                  kRound);
 #if IBUS_CHECK_VERSION(1, 3, 0)
-  ibus_lookup_table_set_orientation(table, IBUS_ORIENTATION_VERTICAL);
+  if (candidates.direction() == commands::Candidates::VERTICAL) {
+    ibus_lookup_table_set_orientation(table, IBUS_ORIENTATION_VERTICAL);
+  } else {
+    ibus_lookup_table_set_orientation(table, IBUS_ORIENTATION_HORIZONTAL);
+  }
 #endif
 
   for (int i = 0; i < candidates.candidate_size(); ++i) {
     const commands::Candidates::Candidate &candidate = candidates.candidate(i);
-    IBusText *text =
-        ibus_text_new_from_string(candidate.value().c_str());
+
+#ifdef OS_CHROMEOS
+    const bool has_description = candidate.has_annotation() &&
+                                 candidate.annotation().has_description();
+    IBusText *text = NULL;
+    if (has_description) {
+      // |kDelimiter| divides a candidate and an annotation.
+      static const char kDelimiter[] = " ";
+
+      // Append an annotation to a candidate word. Both are separated
+      // by |kDelimiter|.
+      text = ibus_text_new_from_string(
+                 (candidate.value() + kDelimiter +
+                  candidate.annotation().description()).c_str());
+
+      // The candidate window on Chrome OS will know
+      // the start index of an annotation by specific number (e.g. 0x888888).
+      // TODO(nhiroki): We should modify the way when iBus supports annotations.
+      const guint kAnnotationForegroundColor = 0x888888;
+
+      // Insert an attribute. It incidates annotation's
+      // start and end index.
+      const guint start = Util::CharsLen(candidate.value() + kDelimiter);
+      const guint end = start +
+          Util::CharsLen(candidate.annotation().description());
+      ibus_text_append_attribute(text,
+                                 IBUS_ATTR_TYPE_FOREGROUND,
+                                 kAnnotationForegroundColor,
+                                 start,
+                                 end);
+    } else {
+      text = ibus_text_new_from_string(candidate.value().c_str());
+    }
+#else
+    IBusText *text = ibus_text_new_from_string(candidate.value().c_str());
+#endif
+
     ibus_lookup_table_append_candidate(table, text);
     // |text| is released by ibus_engine_update_lookup_table along with |table|.
 
@@ -706,8 +941,8 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
   ibus_engine_update_lookup_table(engine, table, TRUE);
   // |table| is released by ibus_engine_update_lookup_table.
 
-  if (candidates.has_focused_index()) {
-    IBusText *auxiliary_text = ComposeAuxiliaryText(candidates);
+  IBusText *auxiliary_text = ComposeAuxiliaryText(candidates);
+  if (auxiliary_text) {
     ibus_engine_update_auxiliary_text(engine, auxiliary_text, TRUE);
     // |auxiliary_text| is released by ibus_engine_update_auxiliary_text.
   } else {
@@ -728,80 +963,35 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
   return true;
 }
 
-void MozcEngine::UpdateConfig(
-    const gchar *section, const gchar *name, GValue *gvalue) {
 #ifdef OS_CHROMEOS
-  if (!section || !name || !gvalue) {
+void MozcEngine::UpdateConfig(const gchar *section,
+                              const gchar *name,
+#if IBUS_CHECK_VERSION(1, 3, 99)
+                              GVariant *value
+#else
+                              GValue *value
+#endif
+                              ) {
+  if (!section || !name || !value) {
     return;
   }
-  if (strcmp(section, kMozcSectionName) != 0) {
+  if (g_strcmp0(section, kMozcSectionName) != 0) {
     return;
   }
 
   config::Config mozc_config;
-  const google::protobuf::Descriptor *descriptor = mozc_config.GetDescriptor();
-  const google::protobuf::Reflection *reflection = mozc_config.GetReflection();
-  const google::protobuf::FieldDescriptor *field_to_update =
-      descriptor->FindFieldByName(name);
-
-  if (!field_to_update) {
-    LOG(ERROR) << "Unknown config name: " << name;
-    return;
-  }
-
-  // Set |gvalue| to |mozc_config|.
-  switch (field_to_update->cpp_type()) {
-    case google::protobuf::FieldDescriptor::CPPTYPE_ENUM: {
-      // |value| should be G_TYPE_STRING.
-      if (!G_VALUE_HOLDS_STRING(gvalue)) {
-        LOG(ERROR) << "Bad GValue type for " << name;
-        return;
-      }
-      const gchar *string_value = g_value_get_string(gvalue);
-      DCHECK(string_value);
-      const google::protobuf::EnumValueDescriptor *enum_value =
-          descriptor->FindEnumValueByName(string_value);
-      if (!enum_value) {
-        LOG(ERROR) << "Bad GValue value for " << name << ": " << string_value;
-        return;
-      }
-      reflection->SetEnum(&mozc_config, field_to_update, enum_value);
-      VLOG(2) << "setting mozc config: " << name << " = " << string_value;
-      break;
-    }
-    case google::protobuf::FieldDescriptor::CPPTYPE_BOOL: {
-      // |value| should be G_TYPE_BOOLEAN
-      if (!G_VALUE_HOLDS_BOOLEAN(gvalue)) {
-        LOG(ERROR) << "Bad GValue type for " << name;
-        return;
-      }
-      const gboolean boolean_value = g_value_get_boolean(gvalue);
-      reflection->SetBool(&mozc_config, field_to_update, boolean_value);
-      VLOG(2) << "setting mozc config: " << name << " = "
-              << (boolean_value ? "true" : "false");
-      break;
-    }
-    default: {
-      // TODO(yusukes): Support other types.
-      LOG(ERROR) << "Unknown or unsupported type: " << name << ": "
-                 << field_to_update->cpp_type();
-      return;
-    }
-  }
+  session_->GetConfig(&mozc_config);
+  ConfigUtil::SetFieldForName(name, value, &mozc_config);
 
   // Update config1.db.
   session_->SetConfig(mozc_config);
   session_->SyncData();  // TODO(yusukes): remove this call?
   VLOG(2) << "Session::SetConfig() is called: " << name;
-#endif
 }
+#endif
 
-void MozcEngine::UpdateCompositionMode(IBusEngine *engine,
-                                       const commands::Output &output) {
-  if (!output.has_mode()) {
-    return;
-  }
-  const commands::CompositionMode new_composition_mode = output.mode();
+void MozcEngine::UpdateCompositionMode(
+    IBusEngine *engine, const commands::CompositionMode new_composition_mode) {
   if (current_composition_mode_ == new_composition_mode) {
     return;
   }
@@ -838,5 +1028,24 @@ void MozcEngine::SyncData(bool force) {
   }
 }
 
+void MozcEngine::RevertSession(IBusEngine *engine) {
+  const commands::CompositionMode original_composition_mode =
+      current_composition_mode_;
+
+  commands::SessionCommand command;
+  command.set_type(commands::SessionCommand::REVERT);
+  commands::Output output;
+  if (!session_->SendCommand(command, &output)) {
+    LOG(ERROR) << "RevertSession() failed";
+    return;
+  }
+  UpdateAll(engine, output);  // may update |current_composition_mode_|.
+
+  // If the original composition mode is DIRECT, we should resume the setting.
+  if (original_composition_mode == commands::DIRECT) {
+    UpdateCompositionMode(engine, original_composition_mode);
+  }
+}
+
 }  // namespace ibus
 }  // namespace mozc
