@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@
 #define MOZC_UNIX_IBUS_MOZC_ENGINE_H_
 
 #include <ibus.h>
+#include <set>
 #include "base/port.h"
 #include "base/scoped_ptr.h"
 #include "session/commands.pb.h"
@@ -94,11 +95,30 @@ class MozcEngine : public EngineInterface {
   // The callback function to the "disconnected" signal to the bus object.
   static void Disconnected(IBusBus *bus, gpointer user_data);
   // The callback function to the "value-changed" signal to the config object.
+#ifdef OS_CHROMEOS
+#if IBUS_CHECK_VERSION(1, 3, 99)
+  static void ConfigValueChanged(IBusConfig *config,
+                                 const gchar *section,
+                                 const gchar *name,
+                                 GVariant *value,
+                                 gpointer user_data);
+#else
   static void ConfigValueChanged(IBusConfig *config,
                                  const gchar *section,
                                  const gchar *name,
                                  GValue *value,
                                  gpointer user_data);
+#endif
+#endif  // OS_CHROMEOS
+
+  // Manages modifier keys.  Returns false if it should not be sent to server.
+  // It is static for unittest.
+  static bool ProcessModifiers(
+      bool is_key_up,
+      gint keyval,
+      commands::KeyEvent *key,
+      set<gint> *currently_pressed_modifiers,
+      set<commands::KeyEvent::ModifierKey> *modifiers_to_be_sent);
 
  private:
   // Updates the preedit text and the candidate window and inserts result
@@ -112,11 +132,19 @@ class MozcEngine : public EngineInterface {
   bool UpdateCandidates(IBusEngine *engine,
                         const commands::Output &output);
   // Updates the configuration.
-  void UpdateConfig(const gchar *section, const gchar *name, GValue *gvalue);
-  // Updates the composition mode based on the content of |output|.
-  void UpdateCompositionMode(IBusEngine *engine,
-                             const commands::Output &output);
+#ifdef OS_CHROMEOS
+#if IBUS_CHECK_VERSION(1, 3, 99)
+  void UpdateConfig(
+      const gchar *section, const gchar *name, GVariant *gvalue);
+#else
+  void UpdateConfig(
+      const gchar *section, const gchar *name, GValue *gvalue);
+#endif
+#endif  // OS_CHROMEOS
 
+  // Updates the composition mode based on the content of |output|.
+  void UpdateCompositionMode(
+      IBusEngine *engine, const commands::CompositionMode new_composition_mode);
   // Updates internal preedit_method (Roman/Kana) state
   void UpdatePreeditMethod();
 
@@ -130,6 +158,10 @@ class MozcEngine : public EngineInterface {
   // always calls SyncData.
   void SyncData(bool force);
 
+  // Reverts internal state of mozc_server by sending SessionCommand::REVERT IPC
+  // message, then hides a preedit string and the candidate window.
+  void RevertSession(IBusEngine *engine);
+
   uint64 last_sync_time_;
   scoped_ptr<KeyTranslator> key_translator_;
   scoped_ptr<client::SessionInterface> session_;
@@ -143,6 +175,15 @@ class MozcEngine : public EngineInterface {
   // Unique IDs of candidates that are currently shown.
   vector<int32> unique_candidate_ids_;
 
+  // Currently pressed modifier keys.  It is set of keyval.
+  set<gint> currently_pressed_modifiers_;
+  // Pending modifier keys.
+  set<commands::KeyEvent::ModifierKey> modifiers_to_be_sent_;
+
+  // A flag to avoid reverting session after deleting surrounding text.
+  // This is a workaround.  See the implementation of ProcessKeyEvent().
+  bool ignore_reset_for_deletion_range_workaround_;
+
   DISALLOW_COPY_AND_ASSIGN(MozcEngine);
 };
 
